@@ -19,14 +19,17 @@
 #ifndef __DEFAULTS_H
 #define __DEFAULTS_H
 
-#define CL_DEFAULT_CFG CONFDIR"/clamav.conf"
+#define CL_DEFAULT_CFG CONFDIR"/clamd.conf"
 #define CL_DEFAULT_BACKLOG 15
-#define CL_DEFAULT_MAXTHREADS 5
+#define CL_DEFAULT_MAXTHREADS 10
 #define CL_DEFAULT_SCANTIMEOUT 120
 #define CL_DEFAULT_LOGSIZE 1048576
 #define CL_DEFAULT_CLAMUKOMAXFILESIZE 5 * 1048576
-#define CL_DEFAULT_SELFCHECK 3600
+#define CL_DEFAULT_SELFCHECK 1800
 #define CL_DEFAULT_MAXWHILEWAIT 120
 #define CL_DEFAULT_MAXPORTSCAN 1000
+#define CL_DEFAULT_MAXDIRREC 15
+#define CL_DEFAULT_STREAMMAXLEN 10 * 1048576
+#define CL_DEFAULT_IDLETIMEOUT 30
 
 #endif
