@@ -29,6 +29,7 @@
 #include <signal.h>
 #include <time.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <pwd.h>
@@ -45,32 +46,33 @@
 #include "output.h"
 #include "target.h"
 #include "misc.h"
+#include "execute.h"
 
 static short terminate = 0;
-
+extern int active_children;
 
 static void daemon_sighandler(int sig) {
-	char *action = NULL;
 
     switch(sig) {
+	case SIGCHLD:
+	    waitpid(-1, NULL, WNOHANG);
+	    active_children--;
+	    break;
+
 	case SIGALRM:
 	case SIGUSR1:
-	    action = "wake up";
 	    terminate = -1;
 	    break;
 
 	case SIGHUP:
-	    action = "re-opening log file";
 	    terminate = -2;
 	    break;
 
 	default:
-	    action = "terminating";
 	    terminate = 1;
 	    break;
     }
 
-    logg("Received signal %d, %s\n", sig, action);
     return;
 }
 
@@ -127,7 +129,7 @@ int freshclam(struct optstruct *opt)
 
     if(cfgopt(copt, "HTTPProxyPassword")) {
 	if(stat(cfgfile, &statbuf) == -1) {
-	    mprintf("@Can't stat %s (critical error)\n");
+	    mprintf("@Can't stat %s (critical error)\n", cfgfile);
 	    return 56;
 	}
 #ifndef C_CYGWIN
@@ -296,23 +298,28 @@ int freshclam(struct optstruct *opt)
 	    writepid(pidfile);
 	}
 
+	active_children = 0;
+
 	logg("freshclam daemon "VERSION" (OS: "TARGET_OS_TYPE", ARCH: "TARGET_ARCH_TYPE", CPU: "TARGET_CPU_TYPE")\n");
 
 	sigaction(SIGTERM, &sigact, NULL);
 	sigaction(SIGHUP, &sigact, NULL);
 	sigaction(SIGINT, &sigact, NULL);
+        sigaction(SIGCHLD, &sigact, NULL);
 
 	while(!terminate) {
 	    ret = download(copt, opt);
 
+            if(ret > 1) {
+		    const char *arg = NULL;
 
-	    if(optl(opt, "on-error-execute")) {
-		if(ret > 1)
-		    system(getargl(opt, "on-error-execute"));
+	        if(optl(opt, "on-error-execute"))
+		    arg = getargl(opt, "on-error-execute");
+		else if((cpt = cfgopt(copt, "OnErrorExecute")))
+		    arg = cpt->strarg;
 
-	    } else if((cpt = cfgopt(copt, "OnErrorExecute"))) {
-		if(ret > 1)
-		    system(cpt->strarg);
+		if(arg)
+		    execute("OnErrorExecute", arg);
 	    }
 
 	    logg("--------------------------------------\n");
@@ -327,8 +334,10 @@ int freshclam(struct optstruct *opt)
 	    } while (!terminate && now < wakeup);
 
 	    if (terminate == -1) {
+		logg("Received signal: wake up\n");
 		terminate = 0;
 	    } else if (terminate == -2) {
+		logg("Received signal: re-opening log file\n");
 		terminate = 0;
 		logg_close();
 	    }
@@ -426,8 +435,8 @@ void help(void)
     mprintf_stdout = 1;
 
     mprintf("\n");
-    mprintf("                          Clam AntiVirus: freshclam  "VERSION"\n");
-    mprintf("                (C) 2002 - 2004 Tomasz Kojm <tkojm@clamav.net>\n\n");
+    mprintf("                   Clam AntiVirus: freshclam  "VERSION"\n");
+    mprintf("    (C) 2002 - 2005 ClamAV Team - http://www.clamav.net/team.html\n\n");
 
     mprintf("    --help               -h              show help\n");
     mprintf("    --version            -V              print version number and exit\n");
@@ -448,6 +457,7 @@ void help(void)
 #ifdef BUILD_CLAMD
     mprintf("    --daemon-notify[=/path/clamd.conf]   send RELOAD command to clamd\n");
 #endif
+    mprintf("    --local-address=IP   -a IP           bind to IP for HTTP downloads\n");
     mprintf("    --on-update-execute=COMMAND          execute COMMAND after successful update\n");
     mprintf("    --on-error-execute=COMMAND           execute COMMAND if errors occured\n");
 
