@@ -38,6 +38,7 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <clamav.h>
+#include <errno.h>
 
 #include "options.h"
 #include "defaults.h"
@@ -45,6 +46,7 @@
 #include "notify.h"
 #include "memory.h"
 #include "output.h"
+#include "misc.h"
 #include "../libclamav/others.h"
 #include "../libclamav/str.h" /* cli_strtok */
 #include "dns.h"
@@ -56,6 +58,8 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	int ret, updated = 0, signo = 0, ttl = -1;
 	char ipaddr[16], *dnsreply = NULL, *pt;
 	struct cfgstruct *cpt;
+	char *localip = NULL;
+	const char *arg = NULL;
 #ifdef HAVE_RESOLV_H
 	const char *dnsdbinfo;
 #endif
@@ -102,17 +106,32 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 		dnsreply = NULL;
 	    }
 
-	    if(dnsreply && (pt = cli_strtok(dnsreply, 0, ":"))) {
-		mprintf("*Software version from DNS: %s\n", pt);
-		if(!strstr(cl_retver(), "devel") && !strstr(cl_retver(), "rc")) {
-		    if(strcmp(cl_retver(), pt)) {
-			mprintf("WARNING: Your ClamAV installation is OUTDATED - please update immediately!\n");
-			mprintf("WARNING: Local version: %s Recommended version: %s\n", cl_retver(), pt);
-			logg("WARNING: Your ClamAV installation is OUTDATED - please update immediately!\n");
-			logg("WARNING: Local version: %s Recommended version: %s\n", cl_retver(), pt);
+	    if(dnsreply) {
+		    int vwarning = 1;
+
+		if((pt = cli_strtok(dnsreply, 4, ":"))) {
+		    if(*pt == '0')
+			vwarning = 0;
+
+		    free(pt);
+		}
+
+		if((pt = cli_strtok(dnsreply, 0, ":"))) {
+
+		    mprintf("*Software version from DNS: %s\n", pt);
+
+		    if(vwarning && !strstr(cl_retver(), "devel") && !strstr(cl_retver(), "rc")) {
+			if(strcmp(cl_retver(), pt)) {
+			    mprintf("WARNING: Your ClamAV installation is OUTDATED!\n");
+			    mprintf("WARNING: Local version: %s Recommended version: %s\n", cl_retver(), pt);
+			    mprintf("DON'T PANIC! Read http://www.clamav.net/faq.html\n");
+			    logg("WARNING: Your ClamAV installation is OUTDATED!\n");
+			    logg("WARNING: Local version: %s Recommended version: %s\n", cl_retver(), pt);
+			    logg("DON'T PANIC! Read http://www.clamav.net/faq.html\n");
+			}
 		    }
+		    free(pt);
 		}
-		free(pt);
 
 	    } else {
 		if(dnsreply) {
@@ -129,9 +148,15 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
     }
 #endif /* HAVE_RESOLV_H */
 
+    if(optl(opt, "localip")) {
+        localip = getargl(opt, "localip");
+    } else if((cpt = cfgopt(copt, "LocalIPAddress"))) {
+	localip = cpt->strarg;
+    }
+
     memset(ipaddr, 0, sizeof(ipaddr));
 
-    if((ret = downloaddb(DB1NAME, "main.cvd", hostname, ipaddr, &signo, copt, dnsreply)) > 50) {
+    if((ret = downloaddb(DB1NAME, "main.cvd", hostname, ipaddr, &signo, copt, dnsreply, localip)) > 50) {
 	if(dnsreply)
 	    free(dnsreply);
 
@@ -141,7 +166,7 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	updated = 1;
 
     /* if ipaddr[0] != 0 it will use it to connect to the web host */
-    if((ret = downloaddb(DB2NAME, "daily.cvd", hostname, ipaddr, &signo, copt, dnsreply)) > 50) {
+    if((ret = downloaddb(DB2NAME, "daily.cvd", hostname, ipaddr, &signo, copt, dnsreply, localip)) > 50) {
 	if(dnsreply)
 	    free(dnsreply);
 
@@ -179,9 +204,16 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 #endif
 
 	if(optl(opt, "on-update-execute"))
-	    system(getargl(opt, "on-update-execute"));
+	    arg = getargl(opt, "on-update-execute");
 	else if((cpt = cfgopt(copt, "OnUpdateExecute")))
-	    system(cpt->strarg);
+	    arg = cpt->strarg;
+
+	if(arg) {
+	    if(optc(opt, 'd'))
+		execute("OnUpdateExecute", arg);
+            else
+		system(arg);
+	}
 
 	return 0;
 
@@ -189,18 +221,7 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	return 1;
 }
 
-static int isnumb(const char *str)
-{
-	int i;
-
-    for(i = 0; i < strlen(str); i++)
-	if(!isdigit(str[i]))
-	    return 0;
-
-    return 1;
-}
-
-int downloaddb(const char *localname, const char *remotename, const char *hostname, char *ip, int *signo, const struct cfgstruct *copt, const char *dnsreply)
+int downloaddb(const char *localname, const char *remotename, const char *hostname, char *ip, int *signo, const struct cfgstruct *copt, const char *dnsreply, char *localip)
 {
 	struct cl_cvd *current, *remote;
 	struct cfgstruct *cpt;
@@ -269,9 +290,9 @@ int downloaddb(const char *localname, const char *remotename, const char *hostna
 
     if(!nodb && dbver == -1) {
 	if(ip[0]) /* use ip to connect */
-	    hostfd = wwwconnect(ip, proxy, port, ipaddr);
+	    hostfd = wwwconnect(ip, proxy, port, ipaddr, localip);
 	else
-	    hostfd = wwwconnect(hostname, proxy, port, ipaddr);
+	    hostfd = wwwconnect(hostname, proxy, port, ipaddr, localip);
 
 	if(hostfd < 0) {
             mprintf("@No servers could be reached. Giving up\n");
@@ -280,6 +301,7 @@ int downloaddb(const char *localname, const char *remotename, const char *hostna
 	    return 52;
 	} else {
 	    mprintf("*Connected to %s (IP: %s).\n", hostname, ipaddr);
+	    mprintf("*Trying to retrieve http://%s/%s\n", hostname, remotename);
 	}
 
 	if(!ip[0])
@@ -314,10 +336,12 @@ int downloaddb(const char *localname, const char *remotename, const char *hostna
 
 	if(flevel < current->fl) {
 	    /* display warning even for already installed database */
-	    mprintf("WARNING: Your ClamAV installation is OUTDATED - please update immediately!\n");
-	    mprintf("WARNING: Current functionality level = %d, required = %d\n", flevel, current->fl);
-	    logg("WARNING: Your ClamAV installation is OUTDATED - please update immediately!\n");
-	    logg("WARNING: Current functionality level = %d, required = %d\n", flevel, current->fl);
+	    mprintf("WARNING: Your ClamAV installation is OUTDATED!\n");
+	    mprintf("WARNING: Current functionality level = %d, recommended = %d\n", flevel, current->fl);
+	    mprintf("DON'T PANIC! Read http://www.clamav.net/faq.html\n");
+	    logg("WARNING: Your ClamAV installation is OUTDATED!\n");
+	    logg("WARNING: Current functionality level = %d, recommended = %d\n", flevel, current->fl);
+	    logg("DON'T PANIC! Read http://www.clamav.net/faq.html\n");
 	}
 
 	*signo += current->sigs;
@@ -330,9 +354,9 @@ int downloaddb(const char *localname, const char *remotename, const char *hostna
 
     if(ipaddr[0]) {
 	/* use ipaddr in order to connect to the same mirror */
-	hostfd = wwwconnect(ipaddr, proxy, port, NULL);
+	hostfd = wwwconnect(ipaddr, proxy, port, NULL, localip);
     } else {
-	hostfd = wwwconnect(hostname, proxy, port, ipaddr);
+	hostfd = wwwconnect(hostname, proxy, port, ipaddr, localip);
 	if(!ip[0])
 	    strcpy(ip, ipaddr);
     }
@@ -350,6 +374,7 @@ int downloaddb(const char *localname, const char *remotename, const char *hostna
      */
     tempname = cli_gentemp(".");
 
+    mprintf("*Retrieving http://%s/%s\n", hostname, remotename);
     if((ret = get_database(remotename, hostfd, tempname, hostname, proxy, user, pass))) {
         mprintf("@Can't download %s from %s (IP: %s)\n", remotename, hostname, ipaddr);
         unlink(tempname);
@@ -388,18 +413,27 @@ int downloaddb(const char *localname, const char *remotename, const char *hostna
 	unlink(tempname);
 	free(tempname);
 	return 53;
-    } else
-	rename(tempname, localname);
-
+    } else {
+    	if(rename(tempname, localname) == -1) {
+    	    mprintf("@Can't rename %s to %s: %s\n", tempname, localname, strerror(errno));
+    	    if(errno == EEXIST) {
+    	        unlink(localname);
+    	        if(rename(tempname, localname) == -1)
+                   mprintf("@All attempts to rename the temporary file failed: %s\n", strerror(errno));
+            }
+        }
+    }
 
     mprintf("%s updated (version: %d, sigs: %d, f-level: %d, builder: %s)\n", localname, current->version, current->sigs, current->fl, current->builder);
     logg("%s updated (version: %d, sigs: %d, f-level: %d, builder: %s)\n", localname, current->version, current->sigs, current->fl, current->builder);
 
     if(flevel < current->fl) {
-	mprintf("WARNING: Your ClamAV installation is OUTDATED - please update immediately!\n");
-	mprintf("WARNING: Current functionality level = %d, required = %d\n", flevel, current->fl);
-	logg("WARNING: Your ClamAV installation is OUTDATED - please update immediately!\n");
-	logg("WARNING: Current functionality level = %d, required = %d\n", flevel, current->fl);
+	mprintf("WARNING: Your ClamAV installation is OUTDATED!\n");
+	mprintf("WARNING: Current functionality level = %d, recommended = %d\n", flevel, current->fl);
+	mprintf("DON'T PANIC! Read http://www.clamav.net/faq.html\n");
+	logg("WARNING: Your ClamAV installation is OUTDATED!\n");
+	logg("WARNING: Current functionality level = %d, recommended = %d\n", flevel, current->fl);
+	logg("DON'T PANIC! Read http://www.clamav.net/faq.html\n");
     }
 
     *signo += current->sigs;
@@ -410,7 +444,7 @@ int downloaddb(const char *localname, const char *remotename, const char *hostna
 
 /* this function returns socket descriptor */
 /* proxy support finshed by njh@bandsman.co.uk */
-int wwwconnect(const char *server, const char *proxy, int pport, char *ip)
+int wwwconnect(const char *server, const char *proxy, int pport, char *ip, char *localip)
 {
 	int socketfd = -1, port, i;
 	struct sockaddr_in name;
@@ -418,13 +452,54 @@ int wwwconnect(const char *server, const char *proxy, int pport, char *ip)
 	char ipaddr[16];
 	unsigned char *ia;
 	const char *hostpt;
-
+	struct hostent *he = NULL;
 
     if(ip)
 	strcpy(ip, "???");
 
     name.sin_family = AF_INET;
 
+    if (localip) {
+	if ((he = gethostbyname(localip)) == NULL) {
+	    char *herr;
+	    switch(h_errno) {
+	        case HOST_NOT_FOUND:
+		    herr = "Host not found";
+		    break;
+
+		case NO_DATA:
+		    herr = "No IP address";
+		    break;
+
+		case NO_RECOVERY:
+		    herr = "Unrecoverable DNS error";
+		    break;
+
+		case TRY_AGAIN:
+		    herr = "Temporary DNS error";
+		    break;
+
+		default:
+		    herr = "Unknown error";
+		    break;
+	    }
+	    mprintf("!Could not resolve local ip address '%s': %s\n", localip, herr);
+	    mprintf("^Using standard local ip address and port for fetching.\n");
+	} else {
+	    struct sockaddr_in client;
+	    memset ((char *) &client, 0, sizeof(struct sockaddr_in));
+	    client.sin_family = AF_INET;
+	    client.sin_addr = *(struct in_addr *) he->h_addr_list[0];
+	    if (bind(socketfd, (struct sockaddr *) &client, sizeof(struct sockaddr_in)) != 0) {
+		mprintf("!Could not bind to local ip address '%s': %s\n", localip, strerror(errno));
+		mprintf("^Using default client ip.\n");
+	    } else {
+		ia = (unsigned char *) he->h_addr_list[0];
+		sprintf(ipaddr, "%u.%u.%u.%u", ia[0], ia[1], ia[2], ia[3]);
+		mprintf("*Using ip '%s' for fetching.\n", ipaddr);
+	    }
+	}
+    }
     if(proxy) {
 	hostpt = proxy;
 
@@ -449,7 +524,29 @@ int wwwconnect(const char *server, const char *proxy, int pport, char *ip)
     }
 
     if((host = gethostbyname(hostpt)) == NULL) {
-        mprintf("@Can't get information about %s host.\n", hostpt);
+	char *herr;
+	switch(h_errno) {
+	    case HOST_NOT_FOUND:
+		herr = "Host not found";
+		break;
+
+	    case NO_DATA:
+		herr = "No IP address";
+		break;
+
+	    case NO_RECOVERY:
+		herr = "Unrecoverable DNS error";
+		break;
+
+	    case TRY_AGAIN:
+		herr = "Temporary DNS error";
+		break;
+
+	    default:
+		herr = "Unknown error";
+		break;
+	}
+        mprintf("@Can't get information about %s: %s\n", hostpt, herr);
 	return -1;
     }
 
