@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002 - 2004 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2002 - 2005 Tomasz Kojm <tkojm@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,6 +43,11 @@ static int targettab[TARGET_TABLE_SIZE] = { 0, CL_TYPE_MSEXE, CL_TYPE_MSOLE2, CL
 
 extern short cli_debug_flag;
 
+#ifdef CL_THREAD_SAFE
+#  include <pthread.h>
+pthread_mutex_t cli_ref_mutex = PTHREAD_MUTEX_INITIALIZER;
+#endif
+
 int cli_scanbuff(const char *buffer, unsigned int length, const char **virname, const struct cl_node *root, unsigned short ftype)
 {
 	int ret, *partcnt;
@@ -125,17 +130,33 @@ static long int cli_caloff(const char *offstr, int fd)
 	}
 	lseek(fd, n, SEEK_SET);
 
-	if(sscanf(offstr, "S%d+%ld", &n, &offset) != 2)
-	    return -1;
+	if(!strncmp(offstr, "SL", 2)) {
 
-	if(n >= peinfo.nsections) {
-	    free(peinfo.section);
-	    return -1;
+	    if(sscanf(offstr, "SL+%ld", &offset) != 1) {
+		free(peinfo.section);
+		return -1;
+	    }
+
+	    offset += peinfo.section[peinfo.nsections - 1].raw;
+
+	} else {
+
+	    if(sscanf(offstr, "S%d+%ld", &n, &offset) != 2) {
+		free(peinfo.section);
+		return -1;
+	    }
+
+	    if(n >= peinfo.nsections) {
+		free(peinfo.section);
+		return -1;
+	    }
+
+	    offset += peinfo.section[n].raw;
 	}
 
-	offset += peinfo.section[n].raw;
 	free(peinfo.section);
 	return offset;
+
     } else if(!strncmp(offstr, "EOF-", 4)) {
 	    struct stat sb;
 
@@ -148,6 +169,40 @@ static long int cli_caloff(const char *offstr, int fd)
     return -1;
 }
 
+int cli_checkfp(int fd, const struct cl_node *root)
+{
+	struct cli_md5_node *md5_node;
+	char *digest;
+
+
+    if(root->md5_hlist) {
+
+	if(!(digest = cli_md5digest(fd))) {
+	    cli_errmsg("cli_checkfp(): Can't generate MD5 checksum\n");
+	    return 0;
+	}
+
+	if((md5_node = cli_vermd5(digest, root)) && md5_node->fp) {
+		struct stat sb;
+
+	    if(fstat(fd, &sb))
+		return CL_EIO;
+
+	    if((unsigned int) sb.st_size != md5_node->size) {
+		cli_warnmsg("Detected false positive MD5 match. Please report.\n");
+	    } else {
+		cli_dbgmsg("Eliminated false positive match (fp sig: %s)\n", md5_node->virname);
+		free(digest);
+		return 1;
+	    }
+	}
+
+	free(digest);
+    }
+
+    return 0;
+}
+
 int cli_validatesig(unsigned short target, unsigned short ftype, const char *offstr, unsigned long int fileoff, int desc, const char *virname)
 {
 
@@ -173,28 +228,12 @@ int cli_validatesig(unsigned short target, unsigned short ftype, const char *off
 	    return 0;
 	}
 
-	if(fileoff != off) {
+	if(fileoff != (unsigned long int) off) {
 	    cli_dbgmsg("Virus offset: %d, expected: %d (%s)\n", fileoff, off, virname);
 	    return 0;
 	}
     }
 
-    if(ftype == CL_TYPE_GRAPHICS && virname && !strncmp(virname, "Exploit.JPEG.Comment", 20)) {
-	    int old;
-
-	if((old = lseek(desc, 0, SEEK_CUR)) == -1) {
-	    cli_dbgmsg("Invalid descriptor\n");
-	    return 0;
-	}
-	lseek(desc, 0, SEEK_SET);
-	if(cli_check_jpeg_exploit(desc) != 1) {
-	    cli_dbgmsg("Eliminated false positive match of Exploit.JPEG.Comment\n");
-	    lseek(desc, old, SEEK_SET);
-	    return 0;
-	}
-	lseek(desc, old, SEEK_SET);
-    }
-
     return 1;
 }
 
@@ -203,7 +242,7 @@ int cli_scandesc(int desc, const char **virname, long int *scanned, const struct
  	char *buffer, *buff, *endbl, *pt;
 	int bytes, buffsize, length, ret, *partcnt, type = CL_CLEAN;
 	unsigned long int *partoff, offset = 0;
-	struct MD5Context ctx;
+	MD5_CTX ctx;
 	unsigned char digest[16];
 	struct cli_md5_node *md5_node;
 
@@ -234,7 +273,7 @@ int cli_scandesc(int desc, const char **virname, long int *scanned, const struct
     }
 
     if(root->md5_hlist)
-	MD5Init(&ctx);
+	MD5_Init(&ctx);
 
 
     buff = buffer;
@@ -258,7 +297,12 @@ int cli_scandesc(int desc, const char **virname, long int *scanned, const struct
 	    free(buffer);
 	    free(partcnt);
 	    free(partoff);
-	    return CL_VIRUS;
+
+	    lseek(desc, 0, SEEK_SET);
+	    if(cli_checkfp(desc, root))
+		return CL_CLEAN;
+	    else
+		return CL_VIRUS;
 
 	} else if(otfrec && ret >= CL_TYPENO) {
 	    if(ret >= type)
@@ -274,7 +318,7 @@ int cli_scandesc(int desc, const char **virname, long int *scanned, const struct
         length = buffsize;
 
 	if(root->md5_hlist)
-	    MD5Update(&ctx, buff, bytes);
+	    MD5_Update(&ctx, buff, bytes);
     }
 
     free(buffer);
@@ -282,7 +326,7 @@ int cli_scandesc(int desc, const char **virname, long int *scanned, const struct
     free(partoff);
 
     if(root->md5_hlist) {
-	MD5Final(digest, &ctx);
+	MD5_Final(digest, &ctx);
 
 	if(cli_debug_flag) {
 		char md5str[33];
@@ -297,13 +341,13 @@ int cli_scandesc(int desc, const char **virname, long int *scanned, const struct
 	    cli_dbgmsg("Calculated MD5 checksum: %s\n", md5str);
 	}
 
-	if((md5_node = cli_vermd5(digest, root))) {
+	if((md5_node = cli_vermd5(digest, root)) && !md5_node->fp) {
 		struct stat sb;
 
 	    if(fstat(desc, &sb))
 		return CL_EIO;
 
-	    if(sb.st_size != md5_node->size) {
+	    if((unsigned int) sb.st_size != md5_node->size) {
 		cli_warnmsg("Detected false positive MD5 match. Please report.\n");
 	    } else {
 		if(virname)
@@ -322,36 +366,92 @@ int cl_build(struct cl_node *root)
     return cli_ac_buildtrie(root);
 }
 
+struct cl_node *cl_dup(struct cl_node *root)
+{
+    if(!root) {
+	cli_errmsg("cl_dup: root == NULL\n");
+	return NULL;
+    }
+
+#ifdef CL_THREAD_SAFE
+    pthread_mutex_lock(&cli_ref_mutex);
+#endif
+
+    root->refcount++;
+    
+#ifdef CL_THREAD_SAFE
+    pthread_mutex_unlock(&cli_ref_mutex);
+#endif
+
+    return root;
+}
+
 void cl_free(struct cl_node *root)
 {
 	int i;
-	struct cli_md5_node *pt, *h;
-
+	struct cli_md5_node *md5pt, *md5h;
+	struct cli_meta_node *metapt, *metah;
 
     if(!root) {
 	cli_errmsg("cl_free: root == NULL\n");
 	return;
     }
 
+#ifdef CL_THREAD_SAFE
+    pthread_mutex_lock(&cli_ref_mutex);
+#endif
+
+    root->refcount--;
+    if (root->refcount) {
+#ifdef CL_THREAD_SAFE
+	pthread_mutex_unlock(&cli_ref_mutex);
+#endif
+	return;
+    }
+    
+#ifdef CL_THREAD_SAFE
+    pthread_mutex_unlock(&cli_ref_mutex);
+#endif
+
     cli_ac_free(root);
     cli_bm_free(root);
 
     if(root->md5_hlist) {
 	for(i = 0; i < 256; i++) {
-	    pt = root->md5_hlist[i];
-	    while(pt) {
-		h = pt;
-		pt = pt->next;
-		free(h->md5);
-		free(h->virname);
-		if(h->viralias)
-		    free(h->viralias);
-		free(h);
+	    md5pt = root->md5_hlist[i];
+	    while(md5pt) {
+		md5h = md5pt;
+		md5pt = md5pt->next;
+		free(md5h->md5);
+		free(md5h->virname);
+		if(md5h->viralias)
+		    free(md5h->viralias);
+		free(md5h);
 	    }
 	}
 	free(root->md5_hlist);
     }
 
+    metapt = root->zip_mlist;
+    while(metapt) {
+	metah = metapt;
+	metapt = metapt->next;
+	free(metah->virname);
+	if(metah->filename)
+	    free(metah->filename);
+	free(metah);
+    }
+
+    metapt = root->rar_mlist;
+    while(metapt) {
+	metah = metapt;
+	metapt = metapt->next;
+	free(metah->virname);
+	if(metah->filename)
+	    free(metah->filename);
+	free(metah);
+    }
+
     free(root);
 }
 
