@@ -14,419 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- * Change History:
- * $Log: message.c,v $
- * Revision 1.143  2005/02/13 09:31:36  nigelhorne
- * Some lines wouldn't uudecode correctly
- *
- * Revision 1.142  2005/02/06 09:22:11  nigelhorne
- * Improved debug statements
- *
- * Revision 1.141  2005/02/03 21:07:34  nigelhorne
- * Faster handling of bounce messages
- *
- * Revision 1.140  2005/02/02 08:35:09  nigelhorne
- * Improved debug message
- *
- * Revision 1.139  2005/01/27 20:20:58  nigelhorne
- * Some Worm.Mydoom.Gen-unp were getting through
- *
- * Revision 1.138  2005/01/24 13:54:22  nigelhorne
- * Some Exploit.IE.CrashSOS were getting through
- *
- * Revision 1.137  2005/01/05 21:54:05  nigelhorne
- * Fuzzy logic lookup of content-type
- *
- * Revision 1.136  2005/01/05 21:07:15  nigelhorne
- * Fix crash when looking for uuencoded attachment fails
- *
- * Revision 1.135  2005/01/01 12:52:58  nigelhorne
- * Some uuencoded viruses were getting through
- *
- * Revision 1.134  2004/12/19 23:19:54  nigelhorne
- * Tidy
- *
- * Revision 1.133  2004/12/19 13:50:08  nigelhorne
- * Tidy
- *
- * Revision 1.132  2004/12/16 15:29:51  nigelhorne
- * Tidy
- *
- * Revision 1.131  2004/12/14 16:45:43  nigelhorne
- * Backtrack quoted-printable broken fix
- *
- * Revision 1.130  2004/12/14 10:27:57  nigelhorne
- * Better reclaiming when running short of memory
- *
- * Revision 1.129  2004/12/10 15:20:23  nigelhorne
- * Handle empty content-type fields
- *
- * Revision 1.128  2004/12/01 12:12:27  nigelhorne
- * Part of rule 3 of paragraph 5.1 of RFC1521 was not being implemented
- *
- * Revision 1.127  2004/11/30 12:03:57  nigelhorne
- * Handle unbalanced quote characters in headers better
- *
- * Revision 1.126  2004/11/28 22:06:39  nigelhorne
- * Tidy space only headers code
- *
- * Revision 1.125  2004/11/28 21:05:49  nigelhorne
- * Handle headers with only spaces
- *
- * Revision 1.124  2004/11/28 16:24:12  nigelhorne
- * Allow lowercase hex characters in quoted-printable
- *
- * Revision 1.123  2004/11/27 21:54:27  nigelhorne
- * Tidy
- *
- * Revision 1.122  2004/11/27 13:16:54  nigelhorne
- * uuencode failures no longer fatal
- *
- * Revision 1.121  2004/11/26 16:58:52  nigelhorne
- * Tidy
- *
- * Revision 1.120  2004/11/26 12:04:22  nigelhorne
- * Fix small typo
- *
- * Revision 1.119  2004/11/22 15:18:51  nigelhorne
- * Performance work
- *
- * Revision 1.118  2004/11/18 18:09:08  nigelhorne
- * First draft of binhex.c
- *
- * Revision 1.117  2004/11/18 10:39:56  nigelhorne
- * Added binhex filetype decoding
- *
- * Revision 1.116  2004/11/17 17:32:15  nigelhorne
- * Find more bounce messages
- *
- * Revision 1.115  2004/11/12 22:21:57  nigelhorne
- * Binxhex detection speeded up
- *
- * Revision 1.114  2004/11/12 09:03:26  nigelhorne
- * Parse some malformed binhex files
- *
- * Revision 1.113  2004/11/11 22:15:46  nigelhorne
- * Rewrite handling of folded headers
- *
- * Revision 1.112  2004/11/09 19:40:06  nigelhorne
- * Find uuencoded files in preambles to multipart messages
- *
- * Revision 1.111  2004/11/08 16:27:09  nigelhorne
- * Fix crash with correctly encoded uuencode files
- *
- * Revision 1.110  2004/11/08 10:26:22  nigelhorne
- * Fix crash if x-yencode is mistakenly guessed
- *
- * Revision 1.109  2004/11/07 16:39:00  nigelhorne
- * Handle para 4 of RFC2231
- *
- * Revision 1.108  2004/10/31 09:28:27  nigelhorne
- * Improve the handling of blank filenames
- *
- * Revision 1.107  2004/10/24 03:51:48  nigelhorne
- * Change encoding guess from warn to debug
- *
- * Revision 1.106  2004/10/22 17:18:13  nigelhorne
- * Handle encoding type us-ascii - should be none
- *
- * Revision 1.105  2004/10/22 15:53:45  nigelhorne
- * Fuzzy logic match for unknown encoding types
- *
- * Revision 1.104  2004/10/19 13:53:55  nigelhorne
- * Don't add trailing NUL bytes
- *
- * Revision 1.103  2004/10/17 09:29:21  nigelhorne
- * Advise to report broken emails
- *
- * Revision 1.102  2004/10/16 20:53:28  nigelhorne
- * Tidy up
- *
- * Revision 1.101  2004/10/16 13:53:52  nigelhorne
- * Handle '8 bit' and plain/text
- *
- * Revision 1.100  2004/10/14 17:45:55  nigelhorne
- * Try to reclaim some memory if it becomes low when decoding
- *
- * Revision 1.99  2004/10/12 10:40:48  nigelhorne
- * Remove shadow declaration of isblank
- *
- * Revision 1.98  2004/10/11 10:56:17  nigelhorne
- * Reimplement squeeze ads sanisiseBase64
- *
- * Revision 1.97  2004/10/06 17:21:46  nigelhorne
- * Code tidy
- *
- * Revision 1.96  2004/10/05 15:46:18  nigelhorne
- * First draft of code to handle RFC1341
- *
- * Revision 1.95  2004/10/05 10:58:00  nigelhorne
- * Table driven base64 decoding
- *
- * Revision 1.94  2004/10/04 12:18:08  nigelhorne
- * Better warning message about PGP attachments not being scanned
- *
- * Revision 1.93  2004/10/01 13:49:22  nigelhorne
- * Minor code tidy
- *
- * Revision 1.92  2004/09/30 08:58:56  nigelhorne
- * Remove empty lines
- *
- * Revision 1.91  2004/09/28 18:39:48  nigelhorne
- * Don't copy if the decoded == the encoded
- *
- * Revision 1.90  2004/09/22 16:24:22  nigelhorne
- * Fix error return
- *
- * Revision 1.89  2004/09/22 16:19:13  nigelhorne
- * Fix error return
- *
- * Revision 1.88  2004/09/21 14:55:26  nigelhorne
- * Handle blank lines in text/plain messages
- *
- * Revision 1.87  2004/09/20 12:44:03  nigelhorne
- * Fix parsing error on mime arguments
- *
- * Revision 1.86  2004/09/18 14:59:26  nigelhorne
- * Code tidy
- *
- * Revision 1.85  2004/09/17 13:47:19  nigelhorne
- * Handle yEnc attachments
- *
- * Revision 1.84  2004/09/17 09:48:53  nigelhorne
- * Handle attempts to hide mime type
- *
- * Revision 1.83  2004/09/16 15:56:45  nigelhorne
- * Handle double colons
- *
- * Revision 1.82  2004/09/16 14:23:57  nigelhorne
- * Handle quotes around mime type
- *
- * Revision 1.81  2004/09/16 12:59:36  nigelhorne
- * Handle = and space as header separaters
- *
- * Revision 1.80  2004/09/16 11:35:08  nigelhorne
- * Minor code tidy
- *
- * Revision 1.79  2004/09/16 10:05:59  nigelhorne
- * Use default decoders
- *
- * Revision 1.78  2004/09/15 18:08:23  nigelhorne
- * Handle multiple encoding types
- *
- * Revision 1.77  2004/09/13 16:44:01  kojm
- * minor cleanup
- *
- * Revision 1.76  2004/09/03 15:59:00  nigelhorne
- * Handle boundary= "foo"
- *
- * Revision 1.75  2004/08/23 13:15:16  nigelhorne
- * messageClearMarkers
- *
- * Revision 1.74  2004/08/22 15:08:59  nigelhorne
- * messageExport
- *
- * Revision 1.73  2004/08/22 10:34:24  nigelhorne
- * Use fileblob
- *
- * Revision 1.72  2004/08/21 11:57:57  nigelhorne
- * Use line.[ch]
- *
- * Revision 1.71  2004/08/13 09:28:16  nigelhorne
- * Remove incorrect comment style
- *
- * Revision 1.70  2004/08/08 19:13:15  nigelhorne
- * Better handling of bounces
- *
- * Revision 1.69  2004/08/04 18:59:19  nigelhorne
- * Tidy up multipart handling
- *
- * Revision 1.68  2004/07/30 11:50:39  nigelhorne
- * Code tidy
- *
- * Revision 1.67  2004/07/26 08:31:04  nigelhorne
- * Fix embedded multi parts
- *
- * Revision 1.66  2004/07/20 15:17:44  nigelhorne
- * Remove overlapping strcpy
- *
- * Revision 1.65  2004/07/20 14:35:29  nigelhorne
- * Some MYDOOM.I were getting through
- *
- * Revision 1.64  2004/07/02 23:00:57  kojm
- * new method of file type detection; HTML normalisation
- *
- * Revision 1.63  2004/06/26 13:16:25  nigelhorne
- * Added newline to end of warning message
- *
- * Revision 1.62  2004/06/24 21:37:26  nigelhorne
- * Handle uuencoded files created with buggy software
- *
- * Revision 1.61  2004/06/22 04:08:02  nigelhorne
- * Optimise empty lines
- *
- * Revision 1.60  2004/06/16 08:07:39  nigelhorne
- * Added thread safety
- *
- * Revision 1.59  2004/06/02 10:11:09  nigelhorne
- * Corrupted binHex could crash on non Linux systems
- *
- * Revision 1.58  2004/06/01 09:07:19  nigelhorne
- * Corrupted binHex could crash on non Linux systems
- *
- * Revision 1.57  2004/05/27 16:52:47  nigelhorne
- * Short binhex data could confuse things
- *
- * Revision 1.56  2004/05/19 10:02:25  nigelhorne
- * Default encoding for attachments set to base64
- *
- * Revision 1.55  2004/05/10 11:24:18  nigelhorne
- * Handle bounce message false positives
- *
- * Revision 1.54  2004/05/06 18:01:25  nigelhorne
- * Force attachments marked as RFC822 messages to be scanned
- *
- * Revision 1.53  2004/04/29 08:59:24  nigelhorne
- * Tidied up SetDispositionType
- *
- * Revision 1.52  2004/04/05 12:04:56  nigelhorne
- * Scan attachments with no filename
- *
- * Revision 1.51  2004/04/01 15:32:34  nigelhorne
- * Graceful exit if messageAddLine fails in strdup
- *
- * Revision 1.50  2004/03/31 17:00:20  nigelhorne
- * Code tidy up free memory earlier
- *
- * Revision 1.49  2004/03/29 09:22:03  nigelhorne
- * Tidy up code and reduce shuffling of data
- *
- * Revision 1.48  2004/03/25 22:40:46  nigelhorne
- * Removed even more calls to realloc and some duplicated code
- *
- * Revision 1.47  2004/03/21 17:19:49  nigelhorne
- * Handle bounce messages with no headers
- *
- * Revision 1.46  2004/03/21 09:41:27  nigelhorne
- * Faster scanning for non MIME messages
- *
- * Revision 1.45  2004/03/20 19:26:48  nigelhorne
- * Second attempt to handle all bounces
- *
- * Revision 1.44  2004/03/20 17:39:23  nigelhorne
- * First attempt to handle all bounces
- *
- * Revision 1.43  2004/03/20 13:23:44  nigelhorne
- * More bounces handled
- *
- * Revision 1.42  2004/03/19 17:38:11  nigelhorne
- * Handle binary encoding as though it had no encoding
- *
- * Revision 1.41  2004/03/19 08:08:38  nigelhorne
- * Handle '8 bit' encoding as well as the RFC '8bit'
- *
- * Revision 1.40  2004/03/18 21:51:41  nigelhorne
- * If a message only contains a single RFC822 message that has no encoding don't save for scanning
- *
- * Revision 1.39  2004/03/18 14:05:25  nigelhorne
- * Added bounce and handle text/plain encoding messages
- *
- * Revision 1.38  2004/03/17 19:47:32  nigelhorne
- * Handle spaces in disposition type
- *
- * Revision 1.37  2004/03/10 05:35:03  nigelhorne
- * Implemented a couple of small speed improvements
- *
- * Revision 1.36  2004/03/07 15:11:48  nigelhorne
- * Fixed minor typo in bounce message
- *
- * Revision 1.35  2004/03/07 12:32:01  nigelhorne
- * Added new bounce message
- *
- * Revision 1.34  2004/02/20 17:04:43  nigelhorne
- * Added new bounce delimeter
- *
- * Revision 1.33  2004/02/18 10:07:40  nigelhorne
- * Find some Yaha
- *
- * Revision 1.32  2004/02/17 20:43:50  nigelhorne
- * Added bounce message
- *
- * Revision 1.31  2004/02/17 09:53:56  nigelhorne
- * Added bounce message
- *
- * Revision 1.30  2004/02/13 14:23:56  nigelhorne
- * Add a new bounce delimeter
- *
- * Revision 1.29  2004/02/10 17:01:30  nigelhorne
- * Recognise a new type of bounce message
- *
- * Revision 1.28  2004/02/07 23:13:55  nigelhorne
- * Handle content-type: text/
- *
- * Revision 1.27  2004/02/06 13:46:08  kojm
- * Support for clamav-config.h
- *
- * Revision 1.26  2004/02/06 13:10:34  nigelhorne
- * Now integrates with winzip
- *
- * Revision 1.25  2004/02/05 11:23:07  nigelhorne
- * Bounce messages are now table driven
- *
- * Revision 1.24  2004/02/04 13:29:16  nigelhorne
- * Handle blobAddData of more than 128K
- *
- * Revision 1.23  2004/02/03 23:04:09  nigelhorne
- * Disabled binhex code
- *
- * Revision 1.22  2004/02/03 22:54:59  nigelhorne
- * Catch another example of Worm.Dumaru.Y
- *
- * Revision 1.21  2004/02/03 14:35:37  nigelhorne
- * Fixed an infinite loop on binhex
- *
- * Revision 1.20  2004/02/02 17:10:04  nigelhorne
- * Scan a rare form of bounce message
- *
- * Revision 1.19  2004/02/02 15:52:09  nigelhorne
- * Remove handling of 8bit binhex files for now
- *
- * Revision 1.18  2004/02/02 15:30:54  nigelhorne
- * Remove handling of 8bit binhex files for now
- *
- * Revision 1.17  2004/02/02 14:01:58  nigelhorne
- * Carefully crafted binhex messages could have caused a crash
- *
- * Revision 1.16  2004/01/28 10:15:24  nigelhorne
- * Added support to scan some bounce messages
- *
- * Revision 1.15  2004/01/14 10:08:45  nigelhorne
- * blobGetData now allows contents to be changed - tuttut
- *
- * Revision 1.14  2004/01/10 13:01:19  nigelhorne
- * Added BinHex compression support
- *
- * Revision 1.13  2004/01/09 18:01:03  nigelhorne
- * Started BinHex work
- *
- * Revision 1.12  2003/12/05 09:34:00  nigelhorne
- * Use cli_tok instead of strtok - replaced now by cli_strtok
- *
- * Revision 1.11  2003/11/17 07:57:12  nigelhorne
- * Prevent buffer overflow in broken uuencoded files
- *
- * Revision 1.10  2003/11/05 07:03:51  nigelhorne
- * Handle broken content-disposition
- *
- * Revision 1.9  2003/10/01 09:28:23  nigelhorne
- * Handle content-type header going over to a new line
- *
- * Revision 1.8  2003/09/28 10:07:08  nigelhorne
- * uuencodebegin() no longer static
- *
  */
-static	char	const	rcsid[] = "$Id: message.c,v 1.143 2005/02/13 09:31:36 nigelhorne Exp $";
+static	char	const	rcsid[] = "$Id: message.c,v 1.152 2005/03/28 11:03:15 nigelhorne Exp $";
 
 #if HAVE_CONFIG_H
 #include "clamav-config.h"
@@ -479,7 +68,6 @@ static	char	const	rcsid[] = "$Id: message.c,v 1.143 2005/02/13 09:31:36 nigelhor
 typedef enum { FALSE = 0, TRUE = 1 } bool;
 
 static	void	messageIsEncoding(message *m);
-static	unsigned char	*decodeLine(message *m, encoding_type enctype, const char *line, unsigned char *buf, size_t buflen);
 static unsigned char *decode(message *m, const char *in, unsigned char *out, unsigned char (*decoder)(char), bool isFast);
 static	void	sanitiseBase64(char *s);
 static	unsigned	char	hex(char c);
@@ -506,7 +94,7 @@ static	const	struct	encoding_map {
 	{	"base64",		BASE64		},	/* rfc2045 */
 	{	"8bit",			EIGHTBIT	},
 	{	"binary",		BINARY		},
-	{	"x-uuencode",		UUENCODE	},
+	{	"x-uuencode",		UUENCODE	},	/* uuencode(5) */
 	{	"x-yencode",		YENCODE		},
 	{	"x-binhex",		BINHEX		},
 	{	"us-ascii",		NOENCODING	},	/* incorrect */
@@ -532,6 +120,9 @@ static	struct	mime_map {
 #define	USE_TABLE	/* table driven base64 decoder */
 
 #ifdef	USE_TABLE
+/*
+ * See RFC2045, section 6.8, table 1
+ */
 static const unsigned char base64Table[256] = {
 	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
 	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
@@ -1256,6 +847,10 @@ messageAddStr(message *m, const char *data)
 		m->body_last = m->body_first = (text *)cli_malloc(sizeof(text));
 	else {
 		assert(m->body_last != NULL);
+		if((data == NULL) && (m->body_last->t_line == NULL))
+			/* don't save two blank lines in sucession */
+			return 1;
+
 		m->body_last->t_next = (text *)cli_malloc(sizeof(text));
 		if(m->body_last->t_next == NULL) {
 			messageDedup(m);
@@ -1354,13 +949,8 @@ messageIsEncoding(message *m)
 		(strncasecmp(line, "Received: ", 10) == 0) &&
 		(cli_filetype(line, strlen(line)) == CL_TYPE_MAIL))
 			m->bounce = m->body_last;
-	else if((m->uuencode == NULL) &&
-		((strncasecmp(line, "begin ", 6) == 0) &&
-		isdigit(line[6]) &&
-		isdigit(line[7]) &&
-		isdigit(line[8]) &&
-		(line[9] == ' ')))
-			m->uuencode = m->body_last;
+	else if((m->uuencode == NULL) && isuuencodebegin(line))
+		m->uuencode = m->body_last;
 	else if((m->binhex == NULL) &&
 		strstr(line, "BinHex") &&
 		(simil(line, binhex) > 90))
@@ -1882,6 +1472,7 @@ messageExport(message *m, const char *dir, void *(*create)(void), void (*destroy
 			}
 
 			if(uptr != data) {
+				assert((size_t)(uptr - data) < datasize);
 				(*addData)(ret, data, (size_t)(uptr - data));
 				size += (size_t)(uptr - data);
 			}
@@ -1903,19 +1494,19 @@ messageExport(message *m, const char *dir, void *(*create)(void), void (*destroy
 		} while((t_line = t_line->t_next) != NULL);
 
 		cli_dbgmsg("Exported %u bytes using enctype %d\n", size, enctype);
-	}
 
-	/* Verify we have nothing left to flush out */
-	if(m->base64chars) {
-		unsigned char data[4];
-		unsigned char *ptr;
+		/* Verify we have nothing left to flush out */
+		if(m->base64chars) {
+			unsigned char data[4];
+			unsigned char *ptr;
 
-		cli_dbgmsg("%u trailing bytes to export\n", m->base64chars);
+			cli_dbgmsg("%u trailing bytes to export\n", m->base64chars);
 
-		ptr = decode(m, NULL, data, base64, FALSE);
-		if(ptr)
-			(*addData)(ret, data, (size_t)(ptr - data));
-		m->base64chars = 0;
+			ptr = decode(m, NULL, data, base64, FALSE);
+			if(ptr)
+				(*addData)(ret, data, (size_t)(ptr - data));
+			m->base64chars = 0;
+		}
 	}
 
 	return ret;
@@ -1991,60 +1582,65 @@ messageToText(message *m)
 
 		cli_dbgmsg("messageToText: export transfer method %d = %d\n",
 			i, enctype);
-		if(enctype == NOENCODING) {
-			/*
-			 * Fast copy
-			 */
-			for(t_line = messageGetBody(m); t_line; t_line = t_line->t_next) {
-				if(first == NULL)
-					first = last = cli_malloc(sizeof(text));
-				else {
-					last->t_next = cli_malloc(sizeof(text));
-					last = last->t_next;
-				}
 
-				if(last == NULL) {
+		switch(enctype) {
+			case NOENCODING:
+			case BINARY:
+			case EIGHTBIT:
+				/*
+				 * Fast copy
+				 */
+				for(t_line = messageGetBody(m); t_line; t_line = t_line->t_next) {
+					if(first == NULL)
+						first = last = cli_malloc(sizeof(text));
+					else {
+						last->t_next = cli_malloc(sizeof(text));
+						last = last->t_next;
+					}
+
+					if(last == NULL) {
+						if(first) {
+							last->t_next = NULL;
+							textDestroy(first);
+						}
+						return NULL;
+					}
+					if(t_line->t_line)
+						last->t_line = lineLink(t_line->t_line);
+					else
+						last->t_line = NULL;	/* empty line */
+				}
+				continue;
+			case UUENCODE:
+				t_line = uuencodeBegin(m);
+
+				if(t_line == NULL) {
+					/*cli_warnmsg("UUENCODED attachment is missing begin statement\n");*/
 					if(first) {
 						last->t_next = NULL;
 						textDestroy(first);
 					}
 					return NULL;
 				}
-				if(t_line->t_line)
-					last->t_line = lineLink(t_line->t_line);
-				else
-					last->t_line = NULL;	/* empty line */
-			}
-			continue;
-		}
-		if(enctype == UUENCODE) {
-			t_line = uuencodeBegin(m);
+				t_line = t_line->t_next;
+				m->uuencode = NULL;
+				break;
+			case YENCODE:
+				t_line = yEncBegin(m);
 
-			if(t_line == NULL) {
-				/*cli_warnmsg("UUENCODED attachment is missing begin statement\n");*/
-				if(first) {
-					last->t_next = NULL;
-					textDestroy(first);
+				if(t_line == NULL) {
+					/*cli_warnmsg("YENCODED attachment is missing begin statement\n");*/
+					if(first) {
+						last->t_next = NULL;
+						textDestroy(first);
+					}
+					return NULL;
 				}
-				return NULL;
-			}
-			t_line = t_line->t_next;
-		} else if(enctype == YENCODE) {
-			t_line = yEncBegin(m);
-
-			if(t_line == NULL) {
-				/*cli_warnmsg("YENCODED attachment is missing begin statement\n");*/
-				if(first) {
-					last->t_next = NULL;
-					textDestroy(first);
-				}
-				return NULL;
-			}
-			t_line = t_line->t_next;
-		} else {
-			if((i == 0) && binhexBegin(m))
-				cli_warnmsg("Binhex messages not supported yet.\n");
-			t_line = messageGetBody(m);
+				t_line = t_line->t_next;
+			default:
+				if((i == 0) && binhexBegin(m))
+					cli_warnmsg("Binhex messages not supported yet.\n");
+				t_line = messageGetBody(m);
 		}
 
 		for(; t_line; t_line = t_line->t_next) {
@@ -2145,16 +1741,9 @@ uuencodeBegin(const message *m)
 	 * lines before the begin. Should not happen, but some
 	 * e-mail clients are rather broken...
 	 */
-	for(t_line = messageGetBody(m); t_line; t_line = t_line->t_next) {
-		const char *line = t_line->t_text;
-
-		if((strncasecmp(line, "begin ", 6) == 0) &&
-		   (isdigit(line[6])) &&
-		   (isdigit(line[7])) &&
-		   (isdigit(line[8])) &&
-		   (line[9] == ' '))
+	for(t_line = messageGetBody(m); t_line; t_line = t_line->t_next)
+		if(isuuencodebegin(t_line->t_text))
 			return t_line;
-	}
 	return NULL;
 }
 #else
@@ -2262,7 +1851,7 @@ messageClearMarkers(message *m)
  *
  * len is sizeof(ptr)
  */
-static unsigned char *
+unsigned char *
 decodeLine(message *m, encoding_type et, const char *line, unsigned char *buf, size_t buflen)
 {
 	size_t len, reallen;
@@ -2362,22 +1951,30 @@ decodeLine(message *m, encoding_type et, const char *line, unsigned char *buf, s
 			break;
 
 		case UUENCODE:
+			assert(m->base64chars == 0);
+
 			if((line == NULL) || (*line == '\0'))	/* empty line */
 				break;
-			if(strncasecmp(line, "begin ", 6) == 0)
-				break;
 			if(strcasecmp(line, "end") == 0)
 				break;
+			if(isuuencodebegin(line))
+				break;
 
 			if((line[0] & 0x3F) == ' ')
 				break;
 
+			/*
+			 * reallen contains the number of bytes that were
+			 *	encoded
+			 */
 			reallen = (size_t)uudecode(*line++);
-			if(reallen == 0)
+			if(reallen <= 0)
+				break;
+			if(reallen > 62)
 				break;
 			len = strlen(line);
 
-			if(len > buflen)
+			if((len > buflen) || (reallen > len))
 				/*
 				 * In practice this should never occur since
 				 * the maximum length of a uuencoded line is
@@ -2388,6 +1985,7 @@ decodeLine(message *m, encoding_type et, const char *line, unsigned char *buf, s
 				(void)decode(m, line, buf, uudecode, (len & 3) == 0);
 				buf = &buf[reallen];
 			}
+			m->base64chars = 0;	/* this happens with broken uuencoded files */
 			break;
 		case YENCODE:
 			if((line == NULL) || (*line == '\0'))	/* empty line */
@@ -2832,7 +2430,14 @@ rfc2231(const char *in)
 				} else
 					*out++ = *in;
 		}
-		in++;
+		if(*in++ == '\0')
+			/*
+			 * Incorrect message that has just one character after
+			 * a '%'.
+			 * FIXME: stash something in out that would, for example
+			 *	treat %2 as %02, assuming field == CONTENTS
+			 */
+			break;
 	}
 
 	if(field != CONTENTS) {
@@ -3028,3 +2633,17 @@ pop(LINK1 *top, char *buffer)
 	}
 	return FAILURE;
 }
+
+/*
+ * Have we found a line that is a start of a uuencoded file (see uuencode(5))?
+ */
+int
+isuuencodebegin(const char *line)
+{
+	if(strlen(line) < 10)
+		return 0;
+
+	return (strncasecmp(line, "begin ", 6) == 0) &&
+		isdigit(line[6]) && isdigit(line[7]) &&
+		isdigit(line[8]) && (line[9] == ' ');
+}
