@@ -62,6 +62,8 @@ pthread_mutex_t cli_gentemp_mutex = PTHREAD_MUTEX_INITIALIZER;
 # else
 #   ifdef	FILENAME_MAX	/* e.g. SCO */
 #     define	NAME_MAX	FILENAME_MAX
+#   else
+#     define	NAME_MAX	256
 #   endif
 # endif
 #endif
@@ -192,21 +194,42 @@ const char *cl_perror(int clerror)
     return cl_strerror(clerror);
 }
 
+unsigned char *cli_md5digest(int desc)
+{
+	unsigned char *digest;
+	char buff[FILEBUFF];
+	MD5_CTX ctx;
+	int bytes;
+
+
+    if(!(digest = cli_malloc(16)))
+	return NULL;
+
+    MD5_Init(&ctx);
+
+    while((bytes = cli_readn(desc, buff, FILEBUFF)))
+	MD5_Update(&ctx, buff, bytes);
+
+    MD5_Final(digest, &ctx);
+
+    return digest;
+}
+
 char *cli_md5stream(FILE *fs, unsigned char *digcpy)
 {
 	unsigned char digest[16];
 	char buff[FILEBUFF];
-	struct MD5Context ctx;
+	MD5_CTX ctx;
 	char *md5str, *pt;
 	int i, bytes;
 
 
-    MD5Init(&ctx);
+    MD5_Init(&ctx);
 
     while((bytes = fread(buff, 1, FILEBUFF, fs)))
-	MD5Update(&ctx, buff, bytes);
+	MD5_Update(&ctx, buff, bytes);
 
-    MD5Final(digest, &ctx);
+    MD5_Final(digest, &ctx);
 
     if(!(md5str = (char *) cli_calloc(32 + 1, sizeof(char))))
 	return NULL;
@@ -244,13 +267,13 @@ static char *cli_md5buff(const char *buffer, unsigned int len)
 {
 	unsigned char digest[16];
 	char *md5str, *pt;
-	struct MD5Context ctx;
+	MD5_CTX ctx;
 	int i;
 
 
-    MD5Init(&ctx);
-    MD5Update(&ctx, buffer, len);
-    MD5Final(digest, &ctx);
+    MD5_Init(&ctx);
+    MD5_Update(&ctx, (unsigned char *) buffer, len);
+    MD5_Final(digest, &ctx);
     memcpy(oldmd5buff, digest, 16);
 
     if(!(md5str = (char *) cli_calloc(32 + 1, sizeof(char))))
