@@ -70,7 +70,7 @@ static inline uint32_t EC32(uint32_t v)
 
 extern short cli_leavetemps_flag;
 
-static uint32_t cli_rawaddr(uint32_t rva, struct pe_image_section_hdr *shp, uint16_t nos)
+static uint32_t cli_rawaddr(uint32_t rva, struct pe_image_section_hdr *shp, uint16_t nos, unsigned int *err)
 {
 	int i, found = 0;
 
@@ -84,12 +84,15 @@ static uint32_t cli_rawaddr(uint32_t rva, struct pe_image_section_hdr *shp, uint
 
     if(!found) {
 	cli_dbgmsg("Can't calculate raw address from RVA 0x%x\n", rva);
-	return -1;
+	*err = 1;
+	return 0;
     }
 
+    *err = 0;
     return rva - EC32(shp[i].VirtualAddress) + EC32(shp[i].PointerToRawData);
 }
 
+/*
 static int cli_ddump(int desc, int offset, int size, const char *file)
 {
 	int pos, ndesc, bread, sum = 0;
@@ -141,8 +144,9 @@ static int cli_ddump(int desc, int offset, int size, const char *file)
     lseek(desc, pos, SEEK_SET);
     return 0;
 }
+*/
 
-int cli_scanpe(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+int cli_scanpe(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	uint16_t e_magic; /* DOS signature ("MZ") */
 	uint16_t nsections;
@@ -154,10 +158,11 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 	struct pe_image_section_hdr *section_hdr;
 	struct stat sb;
 	char sname[9], buff[4096], *tempfile;
-	int i, found, upx_success = 0, min = 0, max = 0, ret;
+	unsigned int i, found, upx_success = 0, min = 0, max = 0, err, broken = 0;
+	unsigned int ssize = 0, dsize = 0, dll = 0;
 	int (*upxfn)(char *, int , char *, int *, uint32_t, uint32_t, uint32_t) = NULL;
 	char *src = NULL, *dest = NULL;
-	int ssize = -1, dsize = -1, ndesc;
+	int ndesc, ret;
 
 
     if(read(desc, &e_magic, sizeof(e_magic)) != sizeof(e_magic)) {
@@ -207,16 +212,28 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 	return CL_CLEAN;
     }
 
+    if(EC16(file_hdr.Characteristics) & 0x2000) {
+	cli_dbgmsg("File type: DLL\n");
+	dll = 1;
+    } else if(EC16(file_hdr.Characteristics) & 0x01) {
+	cli_dbgmsg("File type: Executable\n");
+    }
+
     switch(EC16(file_hdr.Machine)) {
+	case 0x0:
+	    cli_dbgmsg("Machine type: Unknown\n");
 	case 0x14c:
 	    cli_dbgmsg("Machine type: 80386\n");
 	    break;
-	case 0x014d:
+	case 0x14d:
 	    cli_dbgmsg("Machine type: 80486\n");
 	    break;
-	case 0x014e:
+	case 0x14e:
 	    cli_dbgmsg("Machine type: 80586\n");
 	    break;
+	case 0x160:
+	    cli_dbgmsg("Machine type: R30000 (big-endian)\n");
+	    break;
 	case 0x162:
 	    cli_dbgmsg("Machine type: R3000\n");
 	    break;
@@ -253,20 +270,60 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 	case 0x1a2:
 	    cli_dbgmsg("Machine type: Hitachi SH3\n");
 	    break;
+	case 0x1a3:
+	    cli_dbgmsg("Machine type: Hitachi SH3-DSP\n");
+	    break;
+	case 0x1a4:
+	    cli_dbgmsg("Machine type: Hitachi SH3-E\n");
+	    break;
 	case 0x1a6:
 	    cli_dbgmsg("Machine type: Hitachi SH4\n");
 	    break;
+	case 0x1a8:
+	    cli_dbgmsg("Machine type: Hitachi SH5\n");
+	    break;
 	case 0x1c0:
 	    cli_dbgmsg("Machine type: ARM\n");
 	    break;
 	case 0x1c2:
 	    cli_dbgmsg("Machine type: THUMB\n");
 	    break;
+	case 0x1d3:
+	    cli_dbgmsg("Machine type: AM33\n");
+	    break;
+	case 0x520:
+	    cli_dbgmsg("Machine type: Infineon TriCore\n");
+	    break;
+	case 0xcef:
+	    cli_dbgmsg("Machine type: CEF\n");
+	    break;
+	case 0xebc:
+	    cli_dbgmsg("Machine type: EFI Byte Code\n");
+	    break;
+	case 0x9041:
+	    cli_dbgmsg("Machine type: M32R\n");
+	    break;
+	case 0xc0ee:
+	    cli_dbgmsg("Machine type: CEE\n");
+	    break;
+	case 0x8664:
+	    cli_dbgmsg("Machine type: AMD64\n");
+	    break;
 	default:
 	    cli_warnmsg("Unknown machine type in PE header (0x%x)\n", EC16(file_hdr.Machine));
     }
 
     nsections = EC16(file_hdr.NumberOfSections);
+    if(nsections < 1) {
+	if(DETECT_BROKEN) {
+	    if(virname)
+		*virname = "Broken.Executable";
+	    return CL_VIRUS;
+	}
+	cli_warnmsg("PE file contains no sections\n");
+	return CL_CLEAN;
+    }
+
     cli_dbgmsg("NumberOfSections: %d\n", nsections);
 
     timestamp = (time_t) EC32(file_hdr.TimeDateStamp);
@@ -398,17 +455,21 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 
 	if(EC32(section_hdr[i].Characteristics) & 0x20000000)
 	    cli_dbgmsg("Section's memory is executable\n");
+
+	if(EC32(section_hdr[i].Characteristics) & 0x80000000)
+	    cli_dbgmsg("Section's memory is writeable\n");
+
 	cli_dbgmsg("------------------------------------\n");
 
-	if(EC32(section_hdr[i].PointerToRawData) + EC32(section_hdr[i].SizeOfRawData) > sb.st_size) {
+	if(EC32(section_hdr[i].PointerToRawData) + EC32(section_hdr[i].SizeOfRawData) > (unsigned long int) sb.st_size) {
 	    cli_dbgmsg("Possibly broken PE file - Section %d out of file (Offset@ %d, Rsize %d, Total filesize %d)\n", i, EC32(section_hdr[i].PointerToRawData), EC32(section_hdr[i].SizeOfRawData), sb.st_size);
-	    free(section_hdr);
 	    if(DETECT_BROKEN) {
 		if(virname)
 		    *virname = "Broken.Executable";
+		free(section_hdr);
 		return CL_VIRUS;
 	    }
-	    return CL_CLEAN;
+	    broken = 1;
 	}
 
 	if(!i) {
@@ -424,7 +485,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 
     }
 
-    if((ep = EC32(optional_hdr.AddressOfEntryPoint)) >= min && (ep = cli_rawaddr(EC32(optional_hdr.AddressOfEntryPoint), section_hdr, nsections)) == -1) {
+    if((ep = EC32(optional_hdr.AddressOfEntryPoint)) >= min && !(ep = cli_rawaddr(EC32(optional_hdr.AddressOfEntryPoint), section_hdr, nsections, &err)) && err) {
 	cli_dbgmsg("Possibly broken PE file\n");
 	free(section_hdr);
 	if(DETECT_BROKEN) {
@@ -440,7 +501,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
     /* Attempt to detect some popular polymorphic viruses */
 
     /* W32.Parite.B */
-    if(ep == EC32(section_hdr[nsections - 1].PointerToRawData)) {
+    if(!dll && ep == EC32(section_hdr[nsections - 1].PointerToRawData)) {
 	lseek(desc, ep, SEEK_SET);
 	if(read(desc, buff, 4096) == 4096) {
 		const char *pt = cli_memstr(buff, 4040, "\x47\x65\x74\x50\x72\x6f\x63\x41\x64\x64\x72\x65\x73\x73\x00", 15);
@@ -457,11 +518,49 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 	}
     }
 
+    /* W32.Magistr.A/B */
+    if(!dll && (EC32(section_hdr[nsections - 1].Characteristics) & 0x80000000)) {
+	    uint32_t rsize, vsize;
+
+	rsize = EC32(section_hdr[nsections - 1].SizeOfRawData);
+	vsize = EC32(section_hdr[nsections - 1].VirtualSize);
+
+	if(rsize >= 0x612c && vsize >= 0x612c && ((vsize & 0xff) == 0xec)) {
+		int bw = rsize < 0x7000 ? rsize : 0x7000;
+
+	    lseek(desc, EC32(section_hdr[nsections - 1].PointerToRawData) + rsize - bw, SEEK_SET);
+	    if(read(desc, buff, 4096) == 4096) {
+		if(cli_memstr(buff, 4091, "\xe8\x2c\x61\x00\x00", 5)) {
+		    *virname = "W32.Magistr.A";
+		    free(section_hdr);
+		    return CL_VIRUS;
+		} 
+	    }
+
+	} else if(rsize >= 0x7000 && vsize >= 0x7000 && ((vsize & 0xff) == 0xed)) {
+		int bw = rsize < 0x8000 ? rsize : 0x8000;
+
+	    lseek(desc, EC32(section_hdr[nsections - 1].PointerToRawData) + rsize - bw, SEEK_SET);
+	    if(read(desc, buff, 4096) == 4096) {
+		if(cli_memstr(buff, 4091, "\xe8\x04\x72\x00\x00", 5)) {
+		    *virname = "W32.Magistr.B";
+		    free(section_hdr);
+		    return CL_VIRUS;
+		} 
+	    }
+	}
+    }
+
+    if(broken) {
+	free(section_hdr);
+	return CL_CLEAN;
+    }
+
     /* UPX & FSG support */
 
     /* try to find the first section with physical size == 0 */
     found = 0;
-    for(i = 0; i < nsections - 1; i++) {
+    for(i = 0; i < (unsigned int) nsections - 1; i++) {
 	if(!section_hdr[i].SizeOfRawData && section_hdr[i].VirtualSize && section_hdr[i + 1].SizeOfRawData && section_hdr[i + 1].VirtualSize) {
 	    found = 1;
 	    cli_dbgmsg("UPX/FSG: empty section found - assuming compression\n");
@@ -518,14 +617,14 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		}
 
 		lseek(desc, EC32(section_hdr[i + 1].PointerToRawData), SEEK_SET);
-		if(read(desc, src, ssize) != ssize) {
+		if((unsigned int) read(desc, src, ssize) != ssize) {
 		    cli_dbgmsg("Can't read raw data of section %d\n", i);
 		    free(section_hdr);
 		    free(src);
 		    return CL_EIO;
 		}
 
-		if((newedx - EC32(section_hdr[i + 1].VirtualAddress)) < 0 || ((dest = src + newedx - EC32(section_hdr[i + 1].VirtualAddress)) < src && dest >= src + EC32(section_hdr[i + 1].VirtualAddress) + EC32(section_hdr[i + 1].SizeOfRawData) - 4)) {
+		if(newedx < EC32(section_hdr[i + 1].VirtualAddress) || ((dest = src + newedx - EC32(section_hdr[i + 1].VirtualAddress)) < src && dest >= src + EC32(section_hdr[i + 1].VirtualAddress) + EC32(section_hdr[i + 1].SizeOfRawData) - 4)) {
 		    cli_dbgmsg("FSG: New ESP out of bounds\n");
 		    free(src);
 		    break;
@@ -614,7 +713,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		    return CL_CLEAN;
 		}
 
-		if((gp = cli_readint32(buff + 1) - EC32(optional_hdr.ImageBase)) >= EC32(section_hdr[i + 1].PointerToRawData) || gp < 0) {
+		if((gp = cli_readint32(buff + 1) - EC32(optional_hdr.ImageBase)) >= (int) EC32(section_hdr[i + 1].PointerToRawData) || gp < 0) {
 		    cli_dbgmsg("FSG: Support data out of padding area (vaddr: %d)\n", EC32(section_hdr[i].VirtualAddress));
 		    break;
 		}
@@ -622,7 +721,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		lseek(desc, gp, SEEK_SET);
 		gp = EC32(section_hdr[i + 1].PointerToRawData) - gp;
 
-		if(limits && limits->maxfilesize && gp > limits->maxfilesize) {
+		if(limits && limits->maxfilesize && (unsigned int) gp > limits->maxfilesize) {
 		    cli_dbgmsg("FSG: Buffer size exceeded (size: %d, max: %lu)\n", gp, limits->maxfilesize);
 		    free(section_hdr);
 		    return CL_CLEAN;
@@ -644,7 +743,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		newedi = cli_readint32(support + 4) - EC32(optional_hdr.ImageBase); /* 1st dest */
 		newesi = cli_readint32(support + 8) - EC32(optional_hdr.ImageBase); /* Source */
 
-		if(newesi < EC32(section_hdr[i + 1].VirtualAddress || newesi >= EC32(section_hdr[i + 1].VirtualAddress) + EC32(section_hdr[i + 1].SizeOfRawData))) {
+		if(newesi < EC32(section_hdr[i + 1].VirtualAddress) || newesi >= EC32(section_hdr[i + 1].VirtualAddress) + EC32(section_hdr[i + 1].SizeOfRawData)) {
 		    cli_dbgmsg("FSG: Source buffer out of section bounds\n");
 		    free(support);
 		    break;
@@ -700,7 +799,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		}
 
 		lseek(desc, EC32(section_hdr[i + 1].PointerToRawData), SEEK_SET);
-		if(read(desc, src, ssize) != ssize) {
+		if((unsigned int) read(desc, src, ssize) != ssize) {
 		    cli_dbgmsg("Can't read raw data of section %d\n", i);
 		    free(section_hdr);
 		    free(sections);
@@ -802,7 +901,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		}
 		oldep -= EC32(section_hdr[i + 1].VirtualAddress);
 
-		if(newesi < EC32(section_hdr[i + 1].VirtualAddress || newesi >= EC32(section_hdr[i + 1].VirtualAddress) + EC32(section_hdr[i + 1].SizeOfRawData))) {
+		if(newesi < EC32(section_hdr[i + 1].VirtualAddress) || newesi >= EC32(section_hdr[i + 1].VirtualAddress) + EC32(section_hdr[i + 1].SizeOfRawData)) {
 		    cli_dbgmsg("FSG: Source buffer out of section bounds\n");
 		    break;
 		}
@@ -824,7 +923,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		    return CL_CLEAN;
 		}
 
-		if(gp >= EC32(section_hdr[i + 1].PointerToRawData) || gp < 0) {
+		if(gp >= (int) EC32(section_hdr[i + 1].PointerToRawData) || gp < 0) {
 		    cli_dbgmsg("FSG: Support data out of padding area (newedi: %d, vaddr: %d)\n", newedi, EC32(section_hdr[i].VirtualAddress));
 		    break;
 		}
@@ -832,7 +931,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		lseek(desc, gp, SEEK_SET);
 		gp = EC32(section_hdr[i + 1].PointerToRawData) - gp;
 
-		if(limits && limits->maxfilesize && gp > limits->maxfilesize) {
+		if(limits && limits->maxfilesize && (unsigned int) gp > limits->maxfilesize) {
 		    cli_dbgmsg("FSG: Buffer size exceeded (size: %d, max: %lu)\n", gp, limits->maxfilesize);
 		    free(section_hdr);
 		    return CL_CLEAN;
@@ -891,7 +990,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		}
 
 		lseek(desc, EC32(section_hdr[i + 1].PointerToRawData), SEEK_SET);
-		if(read(desc, src, ssize) != ssize) {
+		if((unsigned int) read(desc, src, ssize) != ssize) {
 		    cli_dbgmsg("Can't read raw data of section %d\n", i);
 		    free(section_hdr);
 		    free(sections);
@@ -1017,7 +1116,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 	    }
 
 	    lseek(desc, EC32(section_hdr[i + 1].PointerToRawData), SEEK_SET);
-	    if(read(desc, src, ssize) != ssize) {
+	    if((unsigned int) read(desc, src, ssize) != ssize) {
 		cli_dbgmsg("Can't read raw data of section %d\n", i);
 		free(section_hdr);
 		free(src);
@@ -1123,7 +1222,7 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 		return CL_EIO;
 	    }
 
-	    if(write(ndesc, dest, dsize) != dsize) {
+	    if((unsigned int) write(ndesc, dest, dsize) != dsize) {
 		cli_dbgmsg("UPX/FSG: Can't write %d bytes\n", dsize);
 		free(tempfile);
 		free(dest);
@@ -1166,8 +1265,8 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 	return CL_EIO;
     }
 
-    if(buff[0] != '\xb8' || cli_readint32(buff + 1) != EC32(section_hdr[nsections - 1].VirtualAddress) + EC32(optional_hdr.ImageBase)) {
-	if(buff[0] != '\xb8' || cli_readint32(buff + 1) != EC32(section_hdr[nsections - 2].VirtualAddress) + EC32(optional_hdr.ImageBase))
+    if(buff[0] != '\xb8' || (uint32_t) cli_readint32(buff + 1) != EC32(section_hdr[nsections - 1].VirtualAddress) + EC32(optional_hdr.ImageBase)) {
+	if(nsections < 2 || buff[0] != '\xb8' || (uint32_t) cli_readint32(buff + 1) != EC32(section_hdr[nsections - 2].VirtualAddress) + EC32(optional_hdr.ImageBase))
 	    found = 0;
 	else
 	    found = 1;
@@ -1195,9 +1294,9 @@ int cli_scanpe(int desc, const char **virname, long int *scanned, const struct c
 
 	    for(i = 0 ; i < nsections; i++) {
 		if(section_hdr[i].SizeOfRawData) {
-			uint32_t offset = cli_rawaddr(EC32(section_hdr[i].VirtualAddress), section_hdr, nsections);
+			uint32_t offset = cli_rawaddr(EC32(section_hdr[i].VirtualAddress), section_hdr, nsections, &err);
 
-		    if(offset == -1 || lseek(desc, offset, SEEK_SET) == -1 || read(desc, dest + EC32(section_hdr[i].VirtualAddress) - min, EC32(section_hdr[i].SizeOfRawData)) != EC32(section_hdr[i].SizeOfRawData)) {
+		    if(err || lseek(desc, offset, SEEK_SET) == -1 || (unsigned int) read(desc, dest + EC32(section_hdr[i].VirtualAddress) - min, EC32(section_hdr[i].SizeOfRawData)) != EC32(section_hdr[i].SizeOfRawData)) {
 			free(section_hdr);
 			free(dest);
 			return CL_EIO;
@@ -1275,6 +1374,7 @@ int cli_peheader(int desc, struct cli_pe_info *peinfo)
 	struct pe_image_section_hdr *section_hdr;
 	struct stat sb;
 	int i;
+	unsigned int err;
 
 
     cli_dbgmsg("in cli_peheader\n");
@@ -1369,7 +1469,9 @@ int cli_peheader(int desc, struct cli_pe_info *peinfo)
 	peinfo->section[i].rsz = EC32(section_hdr[i].SizeOfRawData);
     }
 
-    if((peinfo->ep = cli_rawaddr(EC32(optional_hdr.AddressOfEntryPoint), section_hdr, peinfo->nsections)) == -1) {
+    peinfo->ep = cli_rawaddr(EC32(optional_hdr.AddressOfEntryPoint), section_hdr, peinfo->nsections, &err);
+
+    if(err) {
 	cli_dbgmsg("Possibly broken PE file\n");
 	free(section_hdr);
 	free(peinfo->section);
