@@ -52,6 +52,8 @@
 # else
 #   ifdef	FILENAME_MAX	/* e.g. SCO */
 #     define	NAME_MAX	FILENAME_MAX
+#   else
+#     define	NAME_MAX	256
 #   endif
 # endif
 #endif
@@ -62,7 +64,8 @@ static int cli_ac_addsig(struct cl_node *root, const char *virname, const char *
 {
 	struct cli_ac_patt *new;
 	char *pt, *hex;
-	int virlen, ret, i, error = 0;
+	int virlen, ret, error = 0;
+	unsigned int i;
 
 
     if((new = (struct cli_ac_patt *) cli_calloc(1, sizeof(struct cli_ac_patt))) == NULL)
@@ -113,6 +116,7 @@ static int cli_ac_addsig(struct cl_node *root, const char *virname, const char *
 	    new->altn = (unsigned short int *) realloc(new->altn, new->alt * sizeof(unsigned short int));
 	    new->altn[new->alt - 1] = 0;
 	    new->altc = (char **) realloc(new->altc, new->alt * sizeof(char *));
+	    new->altc[new->alt - 1] = NULL;
 
 	    for(i = 0; i < strlen(pt); i++)
 		if(pt[i] == '|')
@@ -161,7 +165,8 @@ static int cli_ac_addsig(struct cl_node *root, const char *virname, const char *
 	    if(new->alt) {
 		free(new->altn);
 		for(i = 0; i < new->alt; i++)
-		    free(new->altc[i]);
+		    if(new->altc[i])
+			free(new->altc[i]);
 		free(new->altc);
 	    }
 	    free(new);
@@ -246,7 +251,8 @@ int cli_parse_add(struct cl_node *root, const char *virname, const char *hexsig,
 {
 	struct cli_bm_patt *bm_new;
 	char *pt, *hexcpy, *start, *n;
-	int ret, virlen, parts = 0, i, j, len, asterisk;
+	int ret, virlen, asterisk = 0;
+	unsigned int i, j, len, parts = 0;
 	int mindist = 0, maxdist = 0, error = 0;
 
 
@@ -433,6 +439,7 @@ static int cli_loaddb(FILE *fd, struct cl_node **root, unsigned int *signo)
 	*root = (struct cl_node *) cli_calloc(1, sizeof(struct cl_node));
 	if(!*root)
 	    return CL_EMEM;
+	(*root)->refcount = 1;
     }
 
     if(!(*root)->ac_root) {
@@ -506,6 +513,7 @@ static int cli_loadndb(FILE *fd, struct cl_node **root, unsigned int *signo)
 	*root = (struct cl_node *) cli_calloc(1, sizeof(struct cl_node));
 	if(!*root)
 	    return CL_EMEM;
+	(*root)->refcount = 1;
     }
 
     if(!(*root)->ac_root) {
@@ -528,6 +536,10 @@ static int cli_loadndb(FILE *fd, struct cl_node **root, unsigned int *signo)
 
     while(fgets(buffer, FILEBUFF, fd)) {
 	line++;
+
+	if(!strncmp(buffer, "Exploit.JPEG.Comment", 20)) /* temporary */
+	    continue;
+
 	sigs++;
 	cli_chomp(buffer);
 
@@ -612,7 +624,7 @@ static int cli_loadndb(FILE *fd, struct cl_node **root, unsigned int *signo)
     return 0;
 }
 
-static int cli_loadhdb(FILE *fd, struct cl_node **root, unsigned int *signo)
+static int cli_loadhdb(FILE *fd, struct cl_node **root, unsigned int *signo, unsigned short fp)
 {
 	char buffer[FILEBUFF], *pt;
 	int line = 0, ret = 0;
@@ -624,6 +636,7 @@ static int cli_loadhdb(FILE *fd, struct cl_node **root, unsigned int *signo)
 	*root = (struct cl_node *) cli_calloc(1, sizeof(struct cl_node));
 	if(!*root)
 	    return CL_EMEM;
+	(*root)->refcount = 1;
     }
 
     while(fgets(buffer, FILEBUFF, fd)) {
@@ -636,6 +649,8 @@ static int cli_loadhdb(FILE *fd, struct cl_node **root, unsigned int *signo)
 	    break;
 	}
 
+	new->fp = fp;
+
 	if(!(pt = cli_strtok(buffer, 0, ":"))) {
 	    free(new);
 	    ret = CL_EMALFDB;
@@ -700,6 +715,180 @@ static int cli_loadhdb(FILE *fd, struct cl_node **root, unsigned int *signo)
     return 0;
 }
 
+static int cli_loadmd(FILE *fd, struct cl_node **root, unsigned int *signo, int type)
+{
+	char buffer[FILEBUFF], *pt;
+	int line = 0, comments = 0, ret = 0;
+	struct cli_meta_node *new;
+
+
+    if(!*root) {
+	cli_dbgmsg("Initializing main node\n");
+	*root = (struct cl_node *) cli_calloc(1, sizeof(struct cl_node));
+	if(!*root)
+	    return CL_EMEM;
+	(*root)->refcount = 1;
+    }
+
+    while(fgets(buffer, FILEBUFF, fd)) {
+	line++;
+	if(buffer[0] == '#') {
+	    comments++;
+	    continue;
+	}
+
+	cli_chomp(buffer);
+
+	new = (struct cli_meta_node *) cli_calloc(1, sizeof(struct cli_meta_node));
+	if(!new) {
+	    ret = CL_EMEM;
+	    break;
+	}
+
+	if(!(new->virname = cli_strtok(buffer, 0, ":"))) {
+	    free(new);
+	    ret = CL_EMALFDB;
+	    break;
+	}
+
+	if(!(pt = cli_strtok(buffer, 1, ":"))) {
+	    free(new->virname);
+	    free(new);
+	    ret = CL_EMALFDB;
+	    break;
+	} else {
+	    new->encrypted = atoi(pt);
+	    free(pt);
+	}
+
+	if(!(new->filename = cli_strtok(buffer, 2, ":"))) {
+	    free(new->virname);
+	    free(new);
+	    ret = CL_EMALFDB;
+	    break;
+	} else {
+	    if(!strcmp(new->filename, "*")) {
+		free(new->filename);
+		new->filename = NULL;
+	    }
+	}
+
+	if(!(pt = cli_strtok(buffer, 3, ":"))) {
+	    free(new->filename);
+	    free(new->virname);
+	    free(new);
+	    ret = CL_EMALFDB;
+	    break;
+	} else {
+	    if(!strcmp(pt, "*"))
+		new->size = -1;
+	    else
+		new->size = atoi(pt);
+	    free(pt);
+	}
+
+	if(!(pt = cli_strtok(buffer, 4, ":"))) {
+	    free(new->filename);
+	    free(new->virname);
+	    free(new);
+	    ret = CL_EMALFDB;
+	    break;
+	} else {
+	    if(!strcmp(pt, "*"))
+		new->csize = -1;
+	    else
+		new->csize = atoi(pt);
+	    free(pt);
+	}
+
+	if(!(pt = cli_strtok(buffer, 5, ":"))) {
+	    free(new->filename);
+	    free(new->virname);
+	    free(new);
+	    ret = CL_EMALFDB;
+	    break;
+	} else {
+	    if(!strcmp(pt, "*")) {
+		new->crc32 = 0;
+	    } else {
+		new->crc32 = cli_hex2num(pt);
+		if(new->crc32 == -1) {
+		    ret = CL_EMALFDB;
+		    break;
+		}
+	    }
+	    free(pt);
+	}
+
+	if(!(pt = cli_strtok(buffer, 6, ":"))) {
+	    free(new->filename);
+	    free(new->virname);
+	    free(new);
+	    ret = CL_EMALFDB;
+	    break;
+	} else {
+	    if(!strcmp(pt, "*"))
+		new->method = -1;
+	    else
+		new->method = atoi(pt);
+	    free(pt);
+	}
+
+	if(!(pt = cli_strtok(buffer, 7, ":"))) {
+	    free(new->filename);
+	    free(new->virname);
+	    free(new);
+	    ret = CL_EMALFDB;
+	    break;
+	} else {
+	    if(!strcmp(pt, "*"))
+		new->fileno = 0;
+	    else
+		new->fileno = atoi(pt);
+	    free(pt);
+	}
+
+	if(!(pt = cli_strtok(buffer, 8, ":"))) {
+	    free(new->filename);
+	    free(new->virname);
+	    free(new);
+	    ret = CL_EMALFDB;
+	    break;
+	} else {
+	    if(!strcmp(pt, "*"))
+		new->maxdepth = 0;
+	    else
+		new->maxdepth = atoi(pt);
+	    free(pt);
+	}
+
+	if(type == 1) {
+	    new->next = (*root)->zip_mlist;
+	    (*root)->zip_mlist = new;
+	} else {
+	    new->next = (*root)->rar_mlist;
+	    (*root)->rar_mlist = new;
+	}
+    }
+
+    if(!line) {
+	cli_errmsg("Empty database file\n");
+	cl_free(*root);
+	return CL_EMALFDB;
+    }
+
+    if(ret) {
+	cli_errmsg("Problem parsing database at line %d\n", line);
+	cl_free(*root);
+	return ret;
+    }
+
+    if(signo)
+	*signo += (line - comments);
+
+    return 0;
+}
+
 int cl_loaddb(const char *filename, struct cl_node **root, unsigned int *signo)
 {
 	FILE *fd;
@@ -725,11 +914,20 @@ int cl_loaddb(const char *filename, struct cl_node **root, unsigned int *signo)
 	ret = cli_cvdload(fd, root, signo, warn);
 
     } else if(cli_strbcasestr(filename, ".hdb")) {
-	ret = cli_loadhdb(fd, root, signo);
+	ret = cli_loadhdb(fd, root, signo, 0);
+
+    } else if(cli_strbcasestr(filename, ".fp")) {
+	ret = cli_loadhdb(fd, root, signo, 1);
 
     } else if(cli_strbcasestr(filename, ".ndb")) {
 	ret = cli_loadndb(fd, root, signo);
 
+    } else if(cli_strbcasestr(filename, ".zmd")) {
+	ret = cli_loadmd(fd, root, signo, 1);
+
+    } else if(cli_strbcasestr(filename, ".rmd")) {
+	ret = cli_loadmd(fd, root, signo, 2);
+
     } else {
 	cli_dbgmsg("cl_loaddb: unknown extension - assuming old database format\n");
 	ret = cli_loaddb(fd, root, signo);
@@ -779,7 +977,10 @@ int cl_loaddbdir(const char *dirname, struct cl_node **root, unsigned int *signo
 	     cli_strbcasestr(dent->d_name, ".db2")  ||
 	     cli_strbcasestr(dent->d_name, ".db3")  ||
 	     cli_strbcasestr(dent->d_name, ".hdb")  ||
+	     cli_strbcasestr(dent->d_name, ".fp")   ||
 	     cli_strbcasestr(dent->d_name, ".ndb")  ||
+	     cli_strbcasestr(dent->d_name, ".zmd")  ||
+	     cli_strbcasestr(dent->d_name, ".rmd")  ||
 	     cli_strbcasestr(dent->d_name, ".cvd"))) {
 
 		dbfile = (char *) cli_calloc(strlen(dent->d_name) + strlen(dirname) + 2, sizeof(char));
@@ -856,7 +1057,10 @@ int cl_statinidir(const char *dirname, struct cl_stat *dbstat)
 	    cli_strbcasestr(dent->d_name, ".db2")  || 
 	    cli_strbcasestr(dent->d_name, ".db3")  || 
 	    cli_strbcasestr(dent->d_name, ".hdb")  || 
+	    cli_strbcasestr(dent->d_name, ".fp")  || 
 	    cli_strbcasestr(dent->d_name, ".ndb")  || 
+	    cli_strbcasestr(dent->d_name, ".zmd")  || 
+	    cli_strbcasestr(dent->d_name, ".rmd")  || 
 	    cli_strbcasestr(dent->d_name, ".cvd"))) {
 
 		dbstat->no++;
@@ -924,7 +1128,10 @@ int cl_statchkdir(const struct cl_stat *dbstat)
 	    cli_strbcasestr(dent->d_name, ".db2")  || 
 	    cli_strbcasestr(dent->d_name, ".db3")  || 
 	    cli_strbcasestr(dent->d_name, ".hdb")  || 
+	    cli_strbcasestr(dent->d_name, ".fp")  || 
 	    cli_strbcasestr(dent->d_name, ".ndb")  || 
+	    cli_strbcasestr(dent->d_name, ".zmd")  || 
+	    cli_strbcasestr(dent->d_name, ".rmd")  || 
 	    cli_strbcasestr(dent->d_name, ".cvd"))) {
 
                 fname = cli_calloc(strlen(dbstat->dir) + strlen(dent->d_name) + 2, sizeof(char));
