@@ -23,9 +23,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <unistd.h>
 #include <sys/param.h>
 #include <fcntl.h>
 #include <dirent.h>
@@ -91,6 +91,8 @@ extern int cli_mbox(const char *dir, int desc, unsigned int options); /* FIXME *
 # else
 #   ifdef	FILENAME_MAX	/* e.g. SCO */
 #     define	NAME_MAX	FILENAME_MAX
+#   else
+#     define	NAME_MAX	256
 #   endif
 # endif
 #endif
@@ -106,7 +108,7 @@ extern int cli_mbox(const char *dir, int desc, unsigned int options); /* FIXME *
 
 #define MAX_MAIL_RECURSION  15
 
-static int cli_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec);
+static int cli_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec);
 
 
 #ifdef CL_THREAD_SAFE
@@ -117,26 +119,27 @@ static void cli_unlock_mutex(void *mtx)
 }
 #endif
 
-static int cli_scanrar(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanrar(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	FILE *tmp = NULL;
-	int files = 0, fd, ret = CL_CLEAN, afiles;
+	int fd, ret = CL_CLEAN;
+	unsigned int files = 0, encrypted, afiles;
 	ArchiveList_struct *rarlist = NULL;
 	ArchiveList_struct *rarlist_head = NULL;
 	char *rar_data_ptr;
 	unsigned long rar_data_size;
+	struct cli_meta_node *mdata;
 
 
     cli_dbgmsg("in scanrar()\n");
 
-
 #ifdef CL_THREAD_SAFE
     pthread_cleanup_push(cli_unlock_mutex, &cli_scanrar_mutex);
     pthread_mutex_lock(&cli_scanrar_mutex);
     cli_scanrar_inuse = 1;
 #endif
 
-    if(! (afiles = urarlib_list(desc, (ArchiveList_struct *) &rarlist))) {
+    if(!(afiles = urarlib_list(desc, (ArchiveList_struct *) &rarlist))) {
 #ifdef CL_THREAD_SAFE
 	pthread_mutex_unlock(&cli_scanrar_mutex);
 	cli_scanrar_inuse = 0;
@@ -149,8 +152,52 @@ static int cli_scanrar(int desc, const char **virname, long int *scanned, const
     rarlist_head = rarlist;
 
     while(rarlist) {
+
+	files++;
+	encrypted = rarlist->item.Flags & 0x04;
+
+	cli_dbgmsg("RAR: %s, crc32: 0x%x, encrypted: %d, compressed: %u, normal: %u, method: %d, ratio: %d (max: %d)\n", rarlist->item.Name, rarlist->item.FileCRC, encrypted, rarlist->item.PackSize, rarlist->item.UnpSize, rarlist->item.Method, rarlist->item.PackSize ? ((unsigned int) rarlist->item.UnpSize / (unsigned int) rarlist->item.PackSize) : 0, limits ? limits->maxratio : 0);
+
+	/* Scan metadata */
+	mdata = root->rar_mlist;
+	if(mdata) do {
+	    if(mdata->encrypted != encrypted)
+		continue;
+
+	    if(mdata->crc32 && (unsigned int) mdata->crc32 != rarlist->item.FileCRC)
+		continue;
+
+	    if(mdata->csize > 0 && (unsigned int) mdata->csize != rarlist->item.PackSize)
+		continue;
+
+	    if(mdata->size >= 0 && (unsigned int) mdata->size != rarlist->item.UnpSize)
+		continue;
+
+	    if(mdata->method && mdata->method != rarlist->item.Method)
+		continue;
+
+	    if(mdata->fileno && mdata->fileno != files)
+		continue;
+
+	    if(mdata->maxdepth && arec > mdata->maxdepth)
+		continue;
+
+	    /* TODO add support for regex */
+	    /*if(mdata->filename && !strstr(zdirent.d_name, mdata->filename))*/
+	    if(mdata->filename && strcmp(rarlist->item.Name, mdata->filename))
+		continue;
+
+	    break; /* matched */
+
+	} while((mdata = mdata->next));
+
+	if(mdata) {
+	    *virname = mdata->virname;
+	    ret = CL_VIRUS;
+	    break;
+	}
+
 	if(DETECT_ENCRYPTED && (rarlist->item.Flags & 0x04)) {
-	    files++;
 	    cli_dbgmsg("RAR: Encrypted files found in archive.\n");
 	    lseek(desc, 0, SEEK_SET);
 	    ret = cli_scandesc(desc, virname, scanned, root, 0, 0);
@@ -166,7 +213,6 @@ static int cli_scanrar(int desc, const char **virname, long int *scanned, const
 	if((rarlist->item.Flags & 0x03) != 0) {
 	    cli_dbgmsg("RAR: Skipping %s (splitted)\n", rarlist->item.Name);
 	    rarlist = rarlist->next;
-	    files++;
 	    continue;
 	}
 
@@ -184,7 +230,6 @@ static int cli_scanrar(int desc, const char **virname, long int *scanned, const
 	    if(limits->maxfilesize && (rarlist->item.UnpSize > (unsigned int) limits->maxfilesize)) {
 		cli_dbgmsg("RAR: %s: Size exceeded (%u, max: %lu)\n", rarlist->item.Name, (unsigned int) rarlist->item.UnpSize, limits->maxfilesize);
 		rarlist = rarlist->next;
-		files++;
 		if(BLOCKMAX) {
 		    *virname = "RAR.ExceededFileSize";
 		    ret = CL_VIRUS;
@@ -206,7 +251,6 @@ static int cli_scanrar(int desc, const char **virname, long int *scanned, const
 
         if(rarlist->item.FileAttr & RAR_FENTRY_ATTR_DIRECTORY) {
             rarlist = rarlist->next;
-            files++;
             continue;
         }
 
@@ -273,7 +317,6 @@ static int cli_scanrar(int desc, const char **virname, long int *scanned, const
 	fclose(tmp);
 	tmp = NULL;
 	rarlist = rarlist->next;
-	files++;
     }
 
     urarlib_freelist(rarlist_head);
@@ -289,15 +332,18 @@ static int cli_scanrar(int desc, const char **virname, long int *scanned, const
 }
 
 #ifdef HAVE_ZLIB_H
-static int cli_scanzip(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanzip(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	ZZIP_DIR *zdir;
 	ZZIP_DIRENT zdirent;
 	ZZIP_FILE *zfp;
 	FILE *tmp = NULL;
 	char *buff;
-	int fd, bytes, files = 0, ret = CL_CLEAN;
+	int fd, bytes, ret = CL_CLEAN;
+	unsigned long int size = 0;
+	unsigned int files = 0, encrypted;
 	struct stat source;
+	struct cli_meta_node *mdata;
 	zzip_error_t err;
 
 
@@ -319,27 +365,72 @@ static int cli_scanzip(int desc, const char **virname, long int *scanned, const
     }
 
     while(zzip_dir_read(zdir, &zdirent)) {
+	files++;
 
 	if(!zdirent.d_name || !strlen(zdirent.d_name)) { /* Mimail fix */
 	    cli_dbgmsg("Zip: strlen(zdirent.d_name) == %d\n", strlen(zdirent.d_name));
-	    *virname = "Suspected.Zip";
+	    *virname = "Suspect.Zip";
 	    ret = CL_VIRUS;
 	    break;
 	}
 
-	cli_dbgmsg("Zip: %s, compressed: %u, normal: %u, ratio: %d (max: %d)\n", zdirent.d_name, zdirent.d_csize, zdirent.st_size, zdirent.d_csize ? (zdirent.st_size / zdirent.d_csize) : 0, limits ? limits->maxratio : -1 );
+        /* Bit 0: file is encrypted
+	 * Bit 6: Strong encryption was used
+	 * Bit 13: Encrypted central directory
+	 */
+	encrypted = (zdirent.d_flags & 0x2041 != 0);
+
+	cli_dbgmsg("Zip: %s, crc32: 0x%x, encrypted: %d, compressed: %u, normal: %u, method: %d, ratio: %d (max: %d)\n", zdirent.d_name, zdirent.d_crc32, encrypted, zdirent.d_csize, zdirent.st_size, zdirent.d_compr, zdirent.d_csize ? (zdirent.st_size / zdirent.d_csize) : 0, limits ? limits->maxratio : 0);
 
 	if(!zdirent.st_size) {
-	    files++;
 	    if(zdirent.d_crc32) {
 		cli_dbgmsg("Zip: Broken file or modified information in local header part of archive\n");
-		*virname = "Suspected.Zip";
+		*virname = "Exploit.Zip.ModifiedHeaders";
 		ret = CL_VIRUS;
 		break;
 	    }
 	    continue;
 	}
 
+	/* Scan metadata */
+	mdata = root->zip_mlist;
+	if(mdata) do {
+	    if(mdata->encrypted != encrypted)
+		continue;
+
+	    if(mdata->crc32 && mdata->crc32 != (unsigned int) zdirent.d_crc32)
+		continue;
+
+	    if(mdata->csize > 0 && mdata->csize != zdirent.d_csize)
+		continue;
+
+	    if(mdata->size >= 0 && mdata->size != zdirent.st_size)
+		continue;
+
+	    if(mdata->method && mdata->method != (unsigned int) zdirent.d_compr)
+		continue;
+
+	    if(mdata->fileno && mdata->fileno != files)
+		continue;
+
+	    if(mdata->maxdepth && arec > mdata->maxdepth)
+		continue;
+
+	    /* TODO add support for regex */
+	    /*if(mdata->filename && !strstr(zdirent.d_name, mdata->filename))*/
+	    if(mdata->filename && strcmp(zdirent.d_name, mdata->filename))
+		continue;
+
+	    break; /* matched */
+
+	} while((mdata = mdata->next));
+
+	if(mdata) {
+	    *virname = mdata->virname;
+	    ret = CL_VIRUS;
+	    break;
+	}
+
 	/* 
 	 * Workaround for archives created with ICEOWS.
 	 * ZZIP_DIRENT does not contain information on file type
@@ -352,9 +443,8 @@ static int cli_scanzip(int desc, const char **virname, long int *scanned, const
 
 	/* work-around for problematic zips (zziplib crashes with them) */
 	if(zdirent.d_csize <= 0 || zdirent.st_size < 0) {
-	    files++;
 	    cli_dbgmsg("Zip: Malformed archive detected.\n");
-	    *virname = "Suspected.Zip";
+	    *virname = "Suspect.Zip";
 	    ret = CL_VIRUS;
 	    break;
 	}
@@ -365,8 +455,7 @@ static int cli_scanzip(int desc, const char **virname, long int *scanned, const
 	    break;
         }
 
-	if(DETECT_ENCRYPTED && (zdirent.d_flags & 1 )) {
-	    files++;
+	if(DETECT_ENCRYPTED && encrypted) {
 	    cli_dbgmsg("Zip: Encrypted files found in archive.\n");
 	    lseek(desc, 0, SEEK_SET);
 	    ret = cli_scandesc(desc, virname, scanned, root, 0, 0);
@@ -380,9 +469,8 @@ static int cli_scanzip(int desc, const char **virname, long int *scanned, const
 	}
 
 	if(limits) {
-	    if(limits->maxfilesize && (zdirent.st_size > limits->maxfilesize)) {
+	    if(limits->maxfilesize && ((unsigned int) zdirent.st_size > limits->maxfilesize)) {
 		cli_dbgmsg("Zip: %s: Size exceeded (%d, max: %ld)\n", zdirent.d_name, zdirent.st_size, limits->maxfilesize);
-		files++;
 		/* ret = CL_EMAXSIZE; */
 		if(BLOCKMAX) {
 		    *virname = "Zip.ExceededFileSize";
@@ -412,11 +500,14 @@ static int cli_scanzip(int desc, const char **virname, long int *scanned, const
 	/* generate temporary file and get its descriptor */
 	if((tmp = tmpfile()) == NULL) {
 	    cli_dbgmsg("Zip: Can't generate tmpfile().\n");
+	    zzip_file_close(zfp);
 	    ret = CL_ETMPFILE;
 	    break;
 	}
 
+	size = 0;
 	while((bytes = zzip_file_read(zfp, buff, FILEBUFF)) > 0) {
+	    size += bytes;
 	    if(fwrite(buff, 1, bytes, tmp) != (size_t) bytes) {
 		cli_dbgmsg("Zip: Can't write to file.\n");
 		zzip_file_close(zfp);
@@ -429,6 +520,11 @@ static int cli_scanzip(int desc, const char **virname, long int *scanned, const
 
 	zzip_file_close(zfp);
 
+	if(size != zdirent.st_size) {
+	    cli_dbgmsg("Zip: Incorrectly decompressed (%d != %d)\n", size, zdirent.st_size);
+	    ret = CL_EIO;
+	    break;
+	}
 
 	if(fflush(tmp) != 0) {
 	    cli_dbgmsg("Zip: fflush() failed: %s\n", strerror(errno));
@@ -444,12 +540,8 @@ static int cli_scanzip(int desc, const char **virname, long int *scanned, const
 	    ret = CL_VIRUS;
 	    break;
 	} else if(ret == CL_EMALFZIP) {
-	    /* 
-	     * The trick with detection of ZoD only works with higher (>= 5)
-	     * recursion limit level.
-	     */
 	    cli_dbgmsg("Zip: Malformed Zip file, scanning stopped.\n");
-	    *virname = "Suspected.Zip";
+	    *virname = "Suspect.Zip";
 	    ret = CL_VIRUS;
 	    break;
 	}
@@ -458,7 +550,6 @@ static int cli_scanzip(int desc, const char **virname, long int *scanned, const
 	    fclose(tmp);
 	    tmp = NULL;
 	}
-	files++;
     }
 
     zzip_dir_close(zdir);
@@ -471,10 +562,10 @@ static int cli_scanzip(int desc, const char **virname, long int *scanned, const
     return ret;
 }
 
-static int cli_scangzip(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scangzip(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	int fd, bytes, ret = CL_CLEAN;
-	long int size = 0;
+	unsigned long int size = 0;
 	char *buff;
 	FILE *tmp = NULL;
 	gzFile gd;
@@ -506,7 +597,10 @@ static int cli_scangzip(int desc, const char **virname, long int *scanned, const
 	if(limits)
 	    if(limits->maxfilesize && (size + FILEBUFF > limits->maxfilesize)) {
 		cli_dbgmsg("GZip: Size exceeded (stopped at %ld, max: %ld)\n", size, limits->maxfilesize);
-		/* ret = CL_EMAXSIZE; */
+		if(BLOCKMAX) {
+		    *virname = "GZip.ExceededFileSize";
+		    ret = CL_VIRUS;
+		}
 		break;
 	    }
 
@@ -521,6 +615,12 @@ static int cli_scangzip(int desc, const char **virname, long int *scanned, const
 
     free(buff);
     gzclose(gd);
+
+    if(ret == CL_VIRUS) {
+	fclose(tmp);
+	return ret;
+    }
+
     if(fsync(fd) == -1) {
 	cli_dbgmsg("GZip: Can't synchronise descriptor %d\n", fd);
 	fclose(tmp);
@@ -547,11 +647,11 @@ static int cli_scangzip(int desc, const char **virname, long int *scanned, const
 #define BZ2_bzRead bzRead
 #endif
 
-static int cli_scanbzip(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanbzip(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	int fd, bytes, ret = CL_CLEAN, bzerror = 0;
 	short memlim = 0;
-	long int size = 0;
+	unsigned long int size = 0;
 	char *buff;
 	FILE *fs, *tmp = NULL;
 	BZFILE *bfd;
@@ -594,7 +694,10 @@ static int cli_scanbzip(int desc, const char **virname, long int *scanned, const
 	if(limits)
 	    if(limits->maxfilesize && (size + FILEBUFF > limits->maxfilesize)) {
 		cli_dbgmsg("Bzip: Size exceeded (stopped at %ld, max: %ld)\n", size, limits->maxfilesize);
-		/* ret = CL_EMAXSIZE; */
+		if(BLOCKMAX) {
+		    *virname = "BZip.ExceededFileSize";
+		    ret = CL_VIRUS;
+		}
 		break;
 	    }
 
@@ -610,6 +713,13 @@ static int cli_scanbzip(int desc, const char **virname, long int *scanned, const
 
     free(buff);
     BZ2_bzReadClose(&bzerror, bfd);
+
+    if(ret == CL_VIRUS) {
+	fclose(tmp);
+	fclose(fs);
+	return ret;
+    }
+
     if(fsync(fd) == -1) {
 	cli_dbgmsg("Bzip: Synchronisation failed for descriptor %d\n", fd);
 	fclose(tmp);
@@ -628,7 +738,7 @@ static int cli_scanbzip(int desc, const char **virname, long int *scanned, const
 }
 #endif
 
-static int cli_scanszdd(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanszdd(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	int fd, ret = CL_CLEAN;
 	FILE *tmp = NULL, *in;
@@ -671,7 +781,7 @@ static int cli_scanszdd(int desc, const char **virname, long int *scanned, const
     return ret;
 }
 
-static int cli_scanmscab(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanmscab(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	struct mscab_decompressor *cabd = NULL;
 	struct mscabd_cabinet *base, *cab;
@@ -730,7 +840,7 @@ static int cli_scanmscab(int desc, const char **virname, long int *scanned, cons
     return ret;
 }
 
-static int cli_scandir(const char *dirname, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scandir(const char *dirname, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	DIR *dd;
 	struct dirent *dent;
@@ -791,7 +901,7 @@ static int cli_scandir(const char *dirname, const char **virname, long int *scan
     return 0;
 }
 
-static int cli_vba_scandir(const char *dirname, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_vba_scandir(const char *dirname, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	int ret = CL_CLEAN, i, fd, data_len;
 	vba_project_t *vba_project;
@@ -936,7 +1046,7 @@ static int cli_vba_scandir(const char *dirname, const char **virname, long int *
     return ret;
 }
 
-static int cli_scanhtml(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanhtml(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	char *tempname, fullname[1024];
 	int ret=CL_CLEAN, fd;
@@ -947,6 +1057,7 @@ static int cli_scanhtml(int desc, const char **virname, long int *scanned, const
     tempname = cli_gentemp(NULL);
     if(mkdir(tempname, 0700)) {
         cli_dbgmsg("ScanHTML -> Can't create temporary directory %s\n", tempname);
+	free(tempname);
         return CL_ETMPDIR;
     }
 
@@ -1009,7 +1120,7 @@ static int cli_scanhtml(int desc, const char **virname, long int *scanned, const
     return ret;
 }
 
-static int cli_scanole2(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanole2(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	char *dir;
 	int ret = CL_CLEAN;
@@ -1021,6 +1132,7 @@ static int cli_scanole2(int desc, const char **virname, long int *scanned, const
     dir = cli_gentemp(NULL);
     if(mkdir(dir, 0700)) {
 	cli_dbgmsg("OLE2: Can't create temporary directory %s\n", dir);
+	free(dir);
 	return CL_ETMPDIR;
     }
 
@@ -1044,7 +1156,7 @@ static int cli_scanole2(int desc, const char **virname, long int *scanned, const
     return ret;
 }
 
-static int cli_scantar(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scantar(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec, unsigned int posix)
 {
 	char *dir;
 	int ret = CL_CLEAN;
@@ -1056,10 +1168,11 @@ static int cli_scantar(int desc, const char **virname, long int *scanned, const
     dir = cli_gentemp(NULL);
     if(mkdir(dir, 0700)) {
 	cli_errmsg("Tar: Can't create temporary directory %s\n", dir);
+	free(dir);
 	return CL_ETMPDIR;
     }
 
-    if((ret = cli_untar(dir, desc)))
+    if((ret = cli_untar(dir, desc, posix)))
 	cli_dbgmsg("Tar: %s\n", cl_strerror(ret));
     else
 	ret = cli_scandir(dir, virname, scanned, root, limits, options, arec, mrec);
@@ -1071,7 +1184,7 @@ static int cli_scantar(int desc, const char **virname, long int *scanned, const
     return ret;
 }
 
-static int cli_scanbinhex(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanbinhex(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	char *dir;
 	int ret = CL_CLEAN;
@@ -1084,6 +1197,7 @@ static int cli_scanbinhex(int desc, const char **virname, long int *scanned, con
 
     if(mkdir(dir, 0700)) {
 	cli_errmsg("Binhex: Can't create temporary directory %s\n", dir);
+	free(dir);
 	return CL_ETMPDIR;
     }
 
@@ -1099,7 +1213,7 @@ static int cli_scanbinhex(int desc, const char **virname, long int *scanned, con
     return ret;
 }
 
-static int cli_scanmschm(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanmschm(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	char *tempname;
 	int ret = CL_CLEAN;
@@ -1110,6 +1224,7 @@ static int cli_scanmschm(int desc, const char **virname, long int *scanned, cons
     tempname = cli_gentemp(NULL);
     if(mkdir(tempname, 0700)) {
 	cli_dbgmsg("CHM: Can't create temporary directory %s\n", tempname);
+	free(tempname);
 	return CL_ETMPDIR;
     }
 
@@ -1123,7 +1238,7 @@ static int cli_scanmschm(int desc, const char **virname, long int *scanned, cons
     return ret;
 }
 
-static int cli_scanscrenc(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanscrenc(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	char *tempname;
 	int ret = CL_CLEAN;
@@ -1133,6 +1248,7 @@ static int cli_scanscrenc(int desc, const char **virname, long int *scanned, con
     tempname = cli_gentemp(NULL);
     if(mkdir(tempname, 0700)) {
 	cli_dbgmsg("CHM: Can't create temporary directory %s\n", tempname);
+	free(tempname);
 	return CL_ETMPDIR;
     }
 
@@ -1146,7 +1262,7 @@ static int cli_scanscrenc(int desc, const char **virname, long int *scanned, con
     return ret;
 }
 
-static int cli_scanriff(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanriff(int desc, const char **virname)
 {
 	int ret = CL_CLEAN;
 
@@ -1158,13 +1274,49 @@ static int cli_scanriff(int desc, const char **virname, long int *scanned, const
     return ret;
 }
 
-static int cli_scanmail(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanjpeg(int desc, const char **virname)
+{
+	int ret = CL_CLEAN;
+
+    if(cli_check_jpeg_exploit(desc) == 1) {
+	ret = CL_VIRUS;
+	*virname = "Exploit.W32.MS04-028";
+    }
+
+    return ret;
+}
+
+static int cli_scantnef(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
+{
+	int ret;
+	char *dir = cli_gentemp(NULL);
+
+
+    if(mkdir(dir, 0700)) {
+	cli_dbgmsg("Can't create temporary directory for tnef file %s\n", dir);
+	free(dir);
+	return CL_ETMPDIR;
+    }
+
+    ret = cli_tnef(dir, desc);
+
+    if(ret == CL_CLEAN)
+	ret = cli_scandir(dir, virname, scanned, root, limits, options, arec, mrec);
+
+    if(!cli_leavetemps_flag)
+	cli_rmdirs(dir);
+
+    free(dir);
+    return ret;
+}
+
+static int cli_scanmail(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	char *dir;
 	int ret;
 
 
-    cli_dbgmsg("Starting cli_scanmail(), mrec == %d, arec == %d\n", *mrec, *arec);
+    cli_dbgmsg("Starting cli_scanmail(), mrec == %d, arec == %d\n", mrec, arec);
 
     /* generate the temporary directory */
     dir = cli_gentemp(NULL);
@@ -1193,14 +1345,24 @@ static int cli_scanmail(int desc, const char **virname, long int *scanned, const
     return ret;
 }
 
-int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
-	char magic[MAGIC_BUFFER_SIZE + 1];
 	int ret = CL_CLEAN, nret;
 	int bread = 0;
 	cli_file_t type;
+	struct stat sb;
 
 
+    if(fstat(desc, &sb) == -1) {
+	cli_errmsg("Can's fstat descriptor %d\n", desc);
+	return CL_EIO;
+    }
+
+    if(sb.st_size <= 5) {
+	cli_dbgmsg("Small data (%d bytes)\n", sb.st_size);
+	return CL_CLEAN;
+    }
+
     if(!root) {
 	cli_errmsg("CRITICAL: root == NULL\n");
 	return CL_EMALFDB;
@@ -1214,8 +1376,8 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
     }
 
     if(SCAN_ARCHIVE && limits && limits->maxreclevel)
-	if(*arec > limits->maxreclevel) {
-	    cli_dbgmsg("Archive recursion limit exceeded (arec == %d).\n", *arec);
+	if(arec > limits->maxreclevel) {
+	    cli_dbgmsg("Archive recursion limit exceeded (arec == %d).\n", arec);
 	    if(BLOCKMAX) {
 		*virname = "Archive.ExceededRecursionLimit";
 		return CL_VIRUS;
@@ -1224,27 +1386,17 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
 	}
 
     if(SCAN_MAIL)
-	if(*mrec > MAX_MAIL_RECURSION) {
-	    cli_dbgmsg("Mail recursion level exceeded (mrec == %d).\n", *mrec);
+	if(mrec > MAX_MAIL_RECURSION) {
+	    cli_dbgmsg("Mail recursion level exceeded (mrec == %d).\n", mrec);
 	    /* return CL_EMAXREC; */
 	    return CL_CLEAN;
 	}
 
     lseek(desc, 0, SEEK_SET);
-    memset(magic, 0, sizeof(magic));
-
-    if((bread = read(desc, magic, MAGIC_BUFFER_SIZE)) == -1) {
-	cli_dbgmsg("Can't read from descriptor %d\n");
-	return CL_EIO;
-    } else if(bread < 2) {
-	/* short read - no need to do magic */
-	return CL_CLEAN;
-    }
-
+    type = cli_filetype2(desc);
     lseek(desc, 0, SEEK_SET);
-    type = cli_filetype(magic, bread);
 
-    type == CL_TYPE_MAIL ? (*mrec)++ : (*arec)++;
+    type == CL_TYPE_MAIL ? mrec++ : arec++;
 
     switch(type) {
 	case CL_TYPE_RAR:
@@ -1284,6 +1436,11 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
 		ret = cli_scanmail(desc, virname, scanned, root, limits, options, arec, mrec);
 	    break;
 
+	case CL_TYPE_TNEF:
+	    if(SCAN_MAIL)
+		ret = cli_scantnef(desc, virname, scanned, root, limits, options, arec, mrec);
+	    break;
+
 	case CL_TYPE_MSCHM:
 	    if(SCAN_ARCHIVE)
 		ret = cli_scanmschm(desc, virname, scanned, root, limits, options, arec, mrec);
@@ -1294,9 +1451,14 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
 		ret = cli_scanole2(desc, virname, scanned, root, limits, options, arec, mrec);
 	    break;
 
-	case CL_TYPE_TAR:
+	case CL_TYPE_POSIX_TAR:
 	    if(SCAN_ARCHIVE)
-		ret = cli_scantar(desc, virname, scanned, root, limits, options, arec, mrec);
+		ret = cli_scantar(desc, virname, scanned, root, limits, options, arec, mrec, 1);
+	    break;
+
+	case CL_TYPE_OLD_TAR:
+	    if(SCAN_ARCHIVE)
+		ret = cli_scantar(desc, virname, scanned, root, limits, options, arec, mrec, 0);
 	    break;
 
 	case CL_TYPE_BINHEX:
@@ -1309,7 +1471,11 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
 	    break;
 
 	case CL_TYPE_RIFF:
-	    ret = cli_scanriff(desc, virname, scanned, root, limits, options, arec, mrec);
+	    ret = cli_scanriff(desc, virname);
+	    break;
+
+	case CL_TYPE_GRAPHICS:
+	    ret = cli_scanjpeg(desc, virname);
 	    break;
 
 	case CL_TYPE_DATA:
@@ -1328,7 +1494,7 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
 	    break;
     }
 
-    type == CL_TYPE_MAIL ? (*mrec)-- : (*arec)--;
+    type == CL_TYPE_MAIL ? mrec-- : arec--;
 
     if(type != CL_TYPE_DATA && ret != CL_VIRUS) { /* scan the raw file */
 	    int typerec;
@@ -1347,7 +1513,7 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
 	} else if(nret >= CL_TYPENO) {
 	    lseek(desc, 0, SEEK_SET);
 
-	    nret == CL_TYPE_MAIL ? (*mrec)++ : (*arec)++;
+	    nret == CL_TYPE_MAIL ? mrec++ : arec++;
 	    switch(nret) {
 		case CL_TYPE_HTML:
 		    if(SCAN_HTML)
@@ -1361,11 +1527,11 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
 			    return CL_VIRUS;
 		    break;
 	    }
-	    nret == CL_TYPE_MAIL ? (*mrec)-- : (*arec)--;
+	    nret == CL_TYPE_MAIL ? mrec-- : arec--;
 	}
     }
 
-    (*arec)++;
+    arec++;
     lseek(desc, 0, SEEK_SET);
     switch(type) {
 	/* Due to performance reasons all executables were first scanned
@@ -1379,7 +1545,7 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
 	default:
 	    break;
     }
-    (*arec)--;
+    arec--;
 
     if(ret == CL_EFORMAT) {
 	cli_dbgmsg("Descriptor[%d]: %s\n", desc, cl_strerror(CL_EFORMAT));
@@ -1391,12 +1557,10 @@ int cli_magic_scandesc(int desc, const char **virname, long int *scanned, const
 
 int cl_scandesc(int desc, const char **virname, unsigned long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options)
 {
-	int arec = 0, mrec = 0;
-
-    return cli_magic_scandesc(desc, virname, scanned, root, limits, options, &arec, &mrec);
+    return cli_magic_scandesc(desc, virname, scanned, root, limits, options, 0, 0);
 }
 
-static int cli_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, int *arec, int *mrec)
+static int cli_scanfile(const char *filename, const char **virname, unsigned long int *scanned, const struct cl_node *root, const struct cl_limits *limits, unsigned int options, unsigned int arec, unsigned int mrec)
 {
 	int fd, ret;
 
