@@ -21,6 +21,21 @@
  *
  * Change History:
  * $Log: untar.c,v $
+ * Revision 1.25  2005/03/22 21:26:27  kojm
+ * add support for old fashioned tar archives
+ *
+ * Revision 1.24  2005/03/20 18:34:18  nigelhorne
+ * Minor tidy
+ *
+ * Revision 1.23  2005/03/20 09:09:25  nigelhorne
+ * Consolidate NAME_MAX
+ *
+ * Revision 1.22  2005/03/10 08:52:10  nigelhorne
+ * Tidy
+ *
+ * Revision 1.21  2005/02/16 22:19:21  nigelhorne
+ * Check file close
+ *
  * Revision 1.20  2005/02/13 22:25:41  kojm
  * do not try to continue if there's no space on device
  *
@@ -82,7 +97,7 @@
  * First draft
  *
  */
-static	char	const	rcsid[] = "$Id: untar.c,v 1.20 2005/02/13 22:25:41 kojm Exp $";
+static	char	const	rcsid[] = "$Id: untar.c,v 1.25 2005/03/22 21:26:27 kojm Exp $";
 
 #include <stdio.h>
 #include <errno.h>
@@ -100,21 +115,6 @@ static	char	const	rcsid[] = "$Id: untar.c,v 1.20 2005/02/13 22:25:41 kojm Exp $"
 
 #define BLOCKSIZE 512
 
-/* Maximum filenames under various systems */
-#ifndef	NAME_MAX	/* e.g. Linux */
-
-#ifdef	MAXNAMELEN	/* e.g. Solaris */
-#define	NAME_MAX	MAXNAMELEN
-#else
-
-#ifdef	FILENAME_MAX	/* e.g. SCO */
-#define	NAME_MAX	FILENAME_MAX
-#endif
-
-#endif
-
-#endif
-
 #ifndef	O_BINARY
 #define	O_BINARY	0
 #endif
@@ -124,12 +124,12 @@ octal(const char *str)
 {
 	int ret = -1;
 
-	sscanf(str, "%o", (unsigned int *)&ret);
+	(void)sscanf(str, "%o", (unsigned int *)&ret);
 	return ret;
 }
 
 int
-cli_untar(const char *dir, int desc)
+cli_untar(const char *dir, int desc, unsigned int posix)
 {
 	int size = 0;
 	int in_block = 0;
@@ -140,7 +140,7 @@ cli_untar(const char *dir, int desc)
 
 	for(;;) {
 		char block[BLOCKSIZE];
-		const int nread = cli_readn(desc, block, sizeof(block));
+		const int nread = cli_readn(desc, block, (unsigned int)sizeof(block));
 
 		if(!in_block && nread == 0)
 			break;
@@ -172,11 +172,13 @@ cli_untar(const char *dir, int desc)
 				break;
 
 			/* Notice assumption that BLOCKSIZE > 262 */
-			strncpy(magic, block+257, 5);
-			magic[5] = '\0';
-			if(strcmp(magic, "ustar") != 0) {
-				cli_dbgmsg("Incorrect magic string '%s' in tar header\n", magic);
-				return CL_EFORMAT;
+			if(posix) {
+				strncpy(magic, block+257, 5);
+				magic[5] = '\0';
+				if(strcmp(magic, "ustar") != 0) {
+					cli_dbgmsg("Incorrect magic string '%s' in tar header\n", magic);
+					return CL_EFORMAT;
+				}
 			}
 
 			type = block[156];
@@ -277,7 +279,7 @@ cli_untar(const char *dir, int desc)
 			cli_dbgmsg("cli_untar: size = %d\n", size);
 		} else { /* write or continue writing file contents */
 			const int nbytes = size>512? 512:size;
-			const int nwritten = fwrite(block, 1, nbytes, outfile);
+			const int nwritten = fwrite(block, 1, (size_t)nbytes, outfile);
 
 			if(nwritten != nbytes) {
 				cli_errmsg("cli_untar: only wrote %d bytes to file %s (out of disk space?)\n",
@@ -291,6 +293,7 @@ cli_untar(const char *dir, int desc)
 			in_block = 0;
 	}
 	if(outfile)
-		fclose(outfile);
+		return fclose(outfile);
+
 	return 0;
 }
