@@ -1,7 +1,7 @@
 /*
  *  Extract VBA source code for component MS Office Documents
  *
- *  Copyright (C) 2004 trog@uncon.org
+ *  Copyright (C) 2004-2005 trog@uncon.org
  *
  *  This code is based on the OpenOffice and libgsf sources.
  *                  
@@ -122,7 +122,7 @@ static char *get_unicode_name(char *name, int size, int is_mac)
         int i, j;
         char *newname;
 
-	 if (*name == 0 || size <= 0) {
+	if (!name || *name == 0 || size <= 0) {
                 return NULL;
         }
 
@@ -481,6 +481,10 @@ vba_project_t *vba56_dir_read(const char *dir)
 			goto out_error;
 		}
 		length = vba_endian_convert_16(length, is_mac);
+		if (length == 0) {
+			cli_dbgmsg("zero name length\n");
+			goto out_error;
+                }
 		buff = (unsigned char *) cli_malloc(length);
 		if (!buff) {
 			cli_dbgmsg("cli_malloc failed\n");
@@ -757,7 +761,7 @@ static int ppt_unlzw(const char *dir, int fd, uint32_t length)
 	stream.next_in = inbuff;
 	bufflen = stream.avail_in = MIN(length, PPT_LZW_BUFFSIZE);
 	
-	if (cli_readn(fd, inbuff, stream.avail_in) != stream.avail_in) {
+	if (cli_readn(fd, inbuff, stream.avail_in) != (int64_t)stream.avail_in) {
 		close(ofd);
 		return FALSE;
 	}
@@ -785,7 +789,7 @@ static int ppt_unlzw(const char *dir, int fd, uint32_t length)
 		if (stream.avail_in == 0) {
 			stream.next_in = inbuff;
 			bufflen = stream.avail_in = MIN(length, PPT_LZW_BUFFSIZE);
-			if (cli_readn(fd, inbuff, stream.avail_in) != stream.avail_in) {
+			if (cli_readn(fd, inbuff, stream.avail_in) != (int64_t)stream.avail_in) {
 				close(ofd);
 				inflateEnd(&stream);
 				return FALSE;
@@ -795,7 +799,7 @@ static int ppt_unlzw(const char *dir, int fd, uint32_t length)
 		retval = inflate(&stream, Z_NO_FLUSH);
 	} while (retval == Z_OK);
 	
-	if (cli_writen(ofd, outbuff, bufflen) != bufflen) {
+	if (cli_writen(ofd, outbuff, bufflen) != (int64_t)bufflen) {
 		close(ofd);
 		inflateEnd(&stream);
 		return FALSE;
@@ -810,6 +814,7 @@ static char *ppt_stream_iter(int fd)
 	atom_header_t atom_header;
 	uint32_t ole_id;
 	char *out_dir;
+	off_t offset;
 	
 	/* Create a directory to store the extracted OLE2 objects */
 	out_dir = cli_gentemp(NULL);
@@ -826,7 +831,7 @@ static char *ppt_stream_iter(int fd)
 		}
 		ppt_print_atom_header(&atom_header);
 
-		if (atom_header.length <= 0) {
+		if (atom_header.length == 0) {
 			cli_rmdirs(out_dir);
 			free(out_dir);
 			return NULL;
@@ -850,7 +855,13 @@ static char *ppt_stream_iter(int fd)
 			}
 
 		} else {
-			if (lseek(fd, atom_header.length, SEEK_CUR) == -1 ) {
+			offset = lseek(fd, 0, SEEK_CUR);
+			/* Check we don't wrap */
+			if ((offset + (off_t)atom_header.length) < offset) {
+				break;
+			}
+			offset += atom_header.length;
+			if (lseek(fd, offset, SEEK_SET) != offset ) {
 				break;
 			}
 		}
@@ -1463,7 +1474,7 @@ vba_project_t *wm_dir_read(const char *dir)
 	}
 	wm_print_fib(&fib);
 	
-	if (lseek(fd, fib.macro_offset, SEEK_SET) != fib.macro_offset) {
+	if (lseek(fd, fib.macro_offset, SEEK_SET) != (int64_t)fib.macro_offset) {
 		cli_dbgmsg("lseek macro_offset failed\n");
 		close(fd);
 		return NULL;
@@ -1604,7 +1615,7 @@ unsigned char *wm_decrypt_macro(int fd, uint32_t offset, uint32_t len,
 	unsigned char *buff;
 	uint32_t i;
 	
-	if (lseek(fd, offset, SEEK_SET) != offset) {
+	if (lseek(fd, offset, SEEK_SET) != (int64_t)offset) {
 		return NULL;
 	}
 	buff = (unsigned char *) cli_malloc(len);
@@ -1612,7 +1623,7 @@ unsigned char *wm_decrypt_macro(int fd, uint32_t offset, uint32_t len,
 		return NULL;
 	}
 
-	if (cli_readn(fd, buff, len) != len) {
+	if (cli_readn(fd, buff, len) != (int64_t)len) {
 		free(buff);
 		return NULL;
 	}
